/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package calc;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import math.VecMath;

import common.Globals;

public class CalculatorVisualizer<T extends Number> extends JPanel {

	private final String[] HEADERS = new String[]{"","Expression","Values","","Result"};
	private CalculatorModel<T> model;
	private JTable table;
	public CalculatorVisualizer(VecMath<T> math) {
		initGui(math);
	}
	public CalculatorModel<T> getModel() {
		return model;
	}
	public JTable getCalculatorTable() {
		return table;
	}
	private void initGui(VecMath<T> math) {
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		setLayout(gridbag);
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1;
		c.weighty = 1;
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.anchor = GridBagConstraints.WEST;
		JComponent comp = createCalculationSheet(math);
		gridbag.setConstraints(comp,c);
		add(comp);
		setPreferredSize(new Dimension(550,300));
	}
	private JComponent createCalculationSheet(VecMath<T> math) {
		model = new CalculatorModel<T>(HEADERS,math);
		table = new JTable(model);
		table.setRowSelectionAllowed(false);
		table.setColumnSelectionAllowed(false);
		StringEditor se = new StringEditor(true);
		ColorRenderer cr = new ColorRenderer();
		((JTextField)se.getComponent()).addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {
				if(!model.isCleared())
					model.clearResults();
				String val = ((JTextField)e.getSource()).getText();
				int rIdx = table.getEditingRow();
				int cIdx = table.getEditingColumn();
				model.setValueAt(val,rIdx,cIdx);
			}
		});
		TableColumn col = table.getColumnModel().getColumn(1);
		col.setCellEditor(se);
		col.setCellRenderer(cr);
		col = table.getColumnModel().getColumn(2);
		col.setCellEditor(se);
		col.setCellRenderer(cr);
		col = table.getColumnModel().getColumn(4);
		col.setCellEditor(new StringEditor(false));
		col = table.getColumnModel().getColumn(3);
		col.setCellEditor(new ComboBoxEditor(CalculatorModel.MODES));
		table.getColumnModel().getColumn(0).setMaxWidth(25);
		table.getColumnModel().getColumn(1).setPreferredWidth(200);
		table.getColumnModel().getColumn(2).setPreferredWidth(40);
		table.getColumnModel().getColumn(3).setMaxWidth(40);
		table.getColumnModel().getColumn(4).setPreferredWidth(120);
		return new JScrollPane(table);
	}
	private class StringEditor extends DefaultCellEditor {

		public StringEditor(boolean editable) {
			super(new JTextField());
			setClickCountToStart(1);
			JTextField c = (JTextField)getComponent();
			c.setEditable(editable);
			if(editable)
				c.setBackground(Globals.INPUT_BG);
		}
	}
	private class ComboBoxEditor extends DefaultCellEditor {

		public ComboBoxEditor(String[] items) {
			super(new JComboBox(items));
		}
	}
	private class ColorRenderer extends DefaultTableCellRenderer {

		@Override
		public Component getTableCellRendererComponent(JTable t,Object v, boolean sel, 
				boolean b, int row,int col) {
			// TODO Auto-generated method stub
			Component c = super.getTableCellRendererComponent(t,v,sel,b,row,col);
			c.setBackground(Globals.INPUT_BG);
			return c;
		}
	}
	public void stopEditing() {
		int rIdx = table.getEditingRow();
		int cIdx = table.getEditingColumn();
		if(rIdx!=-1&&cIdx!=-1)
			table.getCellEditor(rIdx,cIdx).stopCellEditing();
	}
}
