package lattice;

import math.VecMath;
import math.VecMathFloat;

public class EllipseFloat extends Box2D<Float> {

	public EllipseFloat(Float[] p, Float length, VecMath<Float> math) {
		super(p, length, math);
	}
	@Override
	public boolean containsLocal(Float[] pt){
		if(!super.containsLocal(pt))
			return false;
		float l1 = this.getLength((byte)0)/2;
		float l2 = this.getLength((byte)1)/2;
		float dx = pt[0]-l1;
		float dy = pt[1]-l2;
		return dx*dx/(l1*l1)+dy*dy/(l2*l2)<=1;
	}
	@Override
	public boolean intersect(Ray<Float> ray, Float[] isct) {
		float l1 = this.getLength((byte)0)/2;
		float l2 = this.getLength((byte)1)/2;
		if(ray.getDir()[0]!=0){
			float m = ray.getDir()[1]/ray.getDir()[0];
			float t = -ray.getStart()[0]/ray.getDir()[0];
			float c = ray.getDir()[1]*t+ray.getStart()[1];
			float c2 = m*(l1+this.getCorner()[0])+c-this.getCorner()[1]-l2;
			float ll1 = l1*l1;
			float ll2 = l2*l2;
			float p = 2*m*ll1*c2/(ll2+ll1*m*m);
			float q = ll1*(c2*c2-ll2)/(ll2+ll1*m*m);
			float x1 = (float) (-p/2+Math.sqrt(p*p/4-q));
			float x2 = (float) -(p+x1);
			x1 += l1+getCorner()[0];
			x2 += l1+getCorner()[0];
			float t1 = (x1-ray.getStart()[0])/ray.getDir()[0];
			float t2 = (x2-ray.getStart()[0])/ray.getDir()[0];
			if(Math.max(t1,t2)<0)
				return false;			
			else if(Math.min(t1,t2)>=0)
				isct[0] = Math.min(t1,t2)==t1?x1:x2;
			else
				isct[0] = Math.max(t1,t2)==t1?x1:x2;
			isct[1] = m*isct[0]+c;
		} else {
			float x = ray.getStart()[0]-getCorner()[0]-l1;
			if(x/l1>1)
				return false;
			float y = (float) Math.sqrt(1-x*x/(l1*l1));
			if((y+l2+getCorner()[1]-ray.getStart()[1])/ray.getDir()[1]<0)
				y = -y;
			isct[0] = ray.getStart()[0];
			isct[1] = y+l2+getCorner()[1];
		}
		return true;
	}
	public static void main(String[] s){
		EllipseFloat ef = new EllipseFloat(new Float[]{0f,0f},1f,new VecMathFloat());
		Float[] isct = new Float[2];
		Ray<Float> ray = new Ray<Float>(new Float[]{-1f,0.5f},new Float[]{1f,0f},null);
		ef.intersect(ray, isct);
		System.out.println("------------  "+isct[0]+"  -----------  "+isct[1]);
	}
}


