/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package lattice;
import math.VecMath;

import common.IConverter;

public interface IBox<T extends Number> {

	public boolean intersect(Ray<T> r,T[] isct);
	public boolean localIntersect(Ray<T> ray,T[] isct);
	public byte location(T[] p);
	public byte locationLocal(T[] p);
	public boolean contains(T[] p);
	public boolean containsLocal(T[] p);
	public void constrainLocal(T[] p,T[] ret);
	public void rotate(T[] p,T[] axis,T phi);
	public void rotate(T[] p,T[] axis,T sinphi,T cosphi);
	public void setCenter(T[] c);
	public void setCorner(T[] c);
	public void setLength(T l);
	public void setLength(T l,byte dir);
	public void setLengths(T[] ls);
	public T[] getLengths();
	T getLength(byte dir);
	public T[] getCorner();
	public T[][] getSideVectors();
	public T[][] getBaseVectors();
	public IBox<T> copy();
	public void world2LocalPoint(T[] v,T[] ret);
	public void local2WorldPoint(T[] v,T[] ret);
	public void world2LocalVec(T[] v,T[] ret);
	public void local2WorldVec(T[] v,T[] ret);
	public <S extends Number>IBox<S> convert(IConverter<T,S> c,VecMath<S> smath);
}
