/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package math;
import org.apfloat.Apfloat;
import org.apfloat.ApfloatMath;

import common.Log;

public final class VecMathApfloat extends VecMath<Apfloat> {

	private Apfloat ONE_P;
	public VecMathApfloat(short precision) {
		this(precision,true);
	}
	public VecMathApfloat(short precision,boolean useTables) {
		super(new UtilApfloat(),precision,useTables,true);
		ONE_P = limit(ONE);
	}
	@Override
	public Apfloat add(Apfloat x,Apfloat y) {
		return x.add(y);
	}
	@Override
	public Apfloat sub(Apfloat x,Apfloat y) {
		return x.subtract(y);
	}
	@Override
	public Apfloat mul(Apfloat x,Apfloat y) {
		return x.multiply(y);
	}
	@Override
	public Apfloat mul(Apfloat x,int i) {
		return mul(x,util.cast(i));
	}
	@Override
	public Apfloat div(Apfloat x,Apfloat y) {
		return x.divide(y);
	}
	@Override
	public Apfloat sqrt(Apfloat x) {
		return ApfloatMath.sqrt(x);
	}
	@Override
	public Apfloat sqrt(Apfloat x,int p) {
		if(x.precision()>PRECISION)
			x = util.copy(x,p);
		return ApfloatMath.sqrt(x);
	}
	@Override
	public Apfloat neg(Apfloat x) {
		return x.negate();
	}
	@Override
	public Apfloat cos(Apfloat x) {
		long t = System.currentTimeMillis();
		Apfloat retVal = ApfloatMath.cos(x);
		log("cos",x.precision(),t);
		return retVal;
	}
	@Override
	public Apfloat acos(Apfloat x) {
		long t = System.currentTimeMillis();
		Apfloat retVal = ApfloatMath.acos(x);
		log("acos",x.precision(),t);
		return retVal;
	}
	@Override
	public Apfloat sin(Apfloat x) {
		long t = System.currentTimeMillis();
		Apfloat retVal = ApfloatMath.sin(x);
		log("sin",x.precision(),t);
		return retVal;
	}
	@Override
	public Apfloat asin(Apfloat x) {
		long t = System.currentTimeMillis();
		Apfloat retVal = ApfloatMath.asin(x);
		log("asin",x.precision(),t);
		return retVal;
	}
	@Override
	public Apfloat tan(Apfloat x) {
		long t = System.currentTimeMillis();
		Apfloat retVal = ApfloatMath.tan(x);
		log("tan",x.precision(),t);
		return retVal;
	}
	@Override
	public Apfloat atan(Apfloat x) {
		long t = System.currentTimeMillis();
		Apfloat retVal = ApfloatMath.atan(x);
		log("atan",x.precision(),t);
		return retVal;
	}
	@Override
	public Apfloat log(Apfloat x) {
		long t = System.currentTimeMillis();
		Apfloat retVal = ApfloatMath.log(x);
		log("log",x.precision(),t);
		return retVal;
	}
	private static void log(String f,long p,long t){
		t = System.currentTimeMillis()-t;
		String pr = p==Apfloat.INFINITE?"Infinite":p+"";
		Log.getIstc().logln(f+"(x): Precision = "+pr+", Time = "+t+" ms");		
	}
	@Override
	public byte cmp(Apfloat x,Apfloat y) {
		return (byte)x.compareTo(y);
	}
	@Override
	public Apfloat inv(Apfloat x) {
		return ONE_P.divide(x);
	}
	@Override
	public Apfloat abs(Apfloat x) {
		return ApfloatMath.abs(x);
	}
	@Override
	public Apfloat mod(Apfloat x,Apfloat y) {
		return x.mod(y);
	}
	@Override
	public Apfloat pow(Apfloat x,int ex) {
		return ApfloatMath.pow(x,ex);
	}
	@Override
	public Apfloat pow(Apfloat x,Apfloat y) {
		return ApfloatMath.pow(x,y);
	}
	@Override
	public byte sign(Apfloat x) {
		return (byte)x.signum();
	}
	@Override
	public void setPrecision(int p) {
		super.setPrecision(p);
		ONE_P = limit(ONE);
	}
	private Apfloat limit(Apfloat x) {
		long p = x.precision();
		if(p==Apfloat.INFINITE)
			return x.precision(PRECISION+10);
		return x;
	}
}
