/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import lattice.IBox;
import lattice.IShape;
import lattice.Ray;
import lattice.RayPolygon;
import lattice.ReflectionEngine;
import lattice.ShapeContainer;
import math.VecMath;
import pattern.BaseLightPattern;
import pattern.LightCharPattern;

import common.Globals;
import common.Log;

public final class AnimatePatterns<T extends Number> extends AnimateRotation<T> {

	public static long ANIM_SHOW_TIME = 8000;
	public static final byte ANIM_PAT_VERY_SLOW = 0;
	public final static byte ANIM_PAT_SLOW = 1;
	public final static byte ANIM_PAT_MEDIUM = 2;
	public final static byte ANIM_PAT_FAST = 3;
	public static final byte ANIM_PAT_VERY_FAST = 4;
	public static final byte[] ANIM_PAT_SPEEDS =
		new byte[]{0,1,2,3,4};
	
	private List<BaseLightPattern<T>> patterns;
	boolean run2;
	int idx;
	byte speed;
	public AnimatePatterns(Controller<T> c,int startIdx, short gridSize) {
		super(c);
		stopOnEmptyCache = true;
		skipIfOverloaded = false;
		speed = ANIM_PAT_MEDIUM;
		pause = 25;
		patterns = ctrl.getModel().getLightPatterns();
		Iterator<BaseLightPattern<T>> it = patterns.iterator();
		while(it.hasNext()){
			BaseLightPattern<T> blp = it.next();
			if(blp instanceof LightCharPattern)
				it.remove();
		}
		Collections.sort(patterns,new Comparator<BaseLightPattern<T>>(){
			@Override
			public int compare(BaseLightPattern<T> p1,BaseLightPattern<T> p2) {
				if(p1.getCellSequence().size()<p2.getCellSequence().size())
					return -1;
				if(p1.getCellSequence().size()>p2.getCellSequence().size())
					return 1;
				return 0;
			}
		});
		Collections.sort(patterns,new Comparator<BaseLightPattern<T>>(){
			@Override
			public int compare(BaseLightPattern<T> p1,BaseLightPattern<T> p2) {
				if(p1.getGridSize()<p2.getGridSize())
					return -1;
				if(p1.getGridSize()>p2.getGridSize())
					return 1;
				return 0;
			}
		});
		idx = 0;
		if(startIdx>-1)
			idx = startIdx;
		else if(gridSize>1)
			for (idx = 0;idx<patterns.size();idx++)
				if(patterns.get(idx).getGridSize()==gridSize)
					break;
	}
    public synchronized void run(){
    	Model<T> model = ctrl.getModel();
    	IBox<T> gib = model.getGridInitBound();
    	T[] corner = gib.getCorner().clone();
    	VecMath<T> mt = model.getMath();
		model.setGridRotation(mt.ZERO,mt.getBase((byte)1,(byte)2));
    	long t1 = 0;
    	long t2 = 0;
    	while(run2){
    		if(idx==patterns.size())
    			idx = 0;
    		for(;idx<patterns.size()&&run2;idx++){
    			BaseLightPattern<T> blp = patterns.get(idx);
           		T gap = model.calcGap(blp.getGridSize());
        		T r = model.calcShapeSize(blp.getGridSize());
    			T maxSize = mt.mul(blp.getMaxShapeSize(),gap);
    			maxSize = mt.div(maxSize,mt.util.cast(200));
    			if(mt.cmp(r,maxSize)>0){
    				Log.getIstc().log("Skipped pattern "+
    						blp.getName()+" :"+" Selected radius "+
    						"is greater than max. pattern radius.\n");						
    				continue;
    			}
    			blp.setParameters(corner,gap,r);
    			int curPrec = Globals.PRECISIONS[Globals.IDX_PREC];
    			int maxAc = blp.getAccuracyUpperBound();
    			if(curPrec<maxAc){
    				Log.getIstc().log("Skipped pattern "+
    						blp.getName()+" :"+" Selected precision "+
    						"is to low.\n");
    				continue;
    			}
    			T phi = mt.util.copy(blp.getRotation(),mt.PRECISION);
    			T phiCur = model.getGridRotation();
    			T[] sinCosPhi = model.getMath().sincos(phi);
    			T worldX = model.getWorldBox().getCorner()[0];
    			worldX = mt.add(worldX,model.getWorldBox().getLengths()[0]);
    			T[] start = blp.getStart(worldX,phi,sinCosPhi);
    			T delta = mt.mod(mt.sub(phi,phiCur),mt.PI_OVER_TWO);
    			if(mt.cmp(mt.abs(delta),mt.EPSILON10)<=0){
    				if(mt.sign(delta)>=0)
    					phi = mt.add(phiCur,mt.PI_OVER_TWO);
    				else
    					phi = mt.sub(phiCur,mt.PI_OVER_TWO);
    			} else
    				phi = mt.add(phiCur,delta);
    			calcPath(model,rayCache,start,phi,blp.getGridSize(),speed);
    			int stp = 0;
    			for(int l=0;l<rayCache.size();l++)
    				if(rayCache.get(l).getAttribute()==null)
    					stp++;
    			T steps = mt.util.cast(stp/*rayCache.size()*/);
    			T dPhi = mt.div(mt.sub(phi,phiCur),steps);
    			setPhi(dPhi);
    			for(int k = 0;k<rayCache.size();k++)
    				descr.add("");
    			descr.set(descr.size()-1,blp.getName());
    			t2 = System.currentTimeMillis();
    			if(t2-t1<ANIM_SHOW_TIME)
    				try {
    					Thread.sleep(ANIM_SHOW_TIME-(t2-t1));
    				} catch(InterruptedException e) {}
    			if(model.getGridSize()!=blp.getGridSize()){
    				//TODO should be done with listener. In pattern selector also.
    				byte k = Globals.getIndex(blp.getGridSize(),Globals.GRID_SIZES);
    				Globals.IDX_GRID_SIZE = k;
    				MainMenuBar<T> mb = ctrl.getView().getMainMenuBar();
    				if(mb!=null){
        				if(idx!=-1)
        					mb.setSelectedIdx(k,mb.gridSizeMenu,false);
        				else
        					mb.rebuildGridSizeMenu(blp.getGridSize(),false);
        			}
    				model.setGridSize(blp.getGridSize());						
    			} 
    			run = run2;
    			super.run();
    			t1 = System.currentTimeMillis();
    		}
		}
	}
	private void calcPath(Model<T> model,List<Ray<T>> rays,T[] pos,T phi,
			short size,byte speeeed){
		ShapeContainer<Double> fltCtr = model.getShapeContainerDouble();
		fltCtr.setPoints((short) (size+1));
		VecMath<T> tmath = model.getMath();
		VecMath<Double> dmath = model.getDoubleMath();
		T[] st = model.getRay().getStart();
		Double[] pt1 = tmath.util.cast2DoubleArray(st);
		Double[] pt2 = tmath.util.cast2DoubleArray(pos);
        fltCtr.world2LocalPoint(pt1,pt1);
        double dphi = phi.doubleValue()-model.getGridRotation().doubleValue();
        fltCtr.rotate(dmath.ZERO2D,null,dphi);
        fltCtr.world2LocalPoint(pt2,pt2);
		IShape<Double> shape = fltCtr.getShape();
		shape.setSize(shape.getSize()*1.1f);
		fltCtr.setShape(shape);
		if(fltCtr.getShapeLocal(pt1)!=null||fltCtr.getShapeLocal(pt2)!=null)
			shape.setSize(shape.getSize()/1.1f);	
		List<Double[]> starts = new ArrayList<Double[]>();
    	IBox<T> gib = model.getGridInitBound();
    	double l = gib.getLength((byte)1).doubleValue();
		double dx = (pt2[0]-pt1[0])/l;
		double dy = (pt2[1]-pt1[1])/l;
		double dp = dphi/Math.PI;
		double div = 0.002;
		switch(speeeed){
		case ANIM_PAT_VERY_SLOW :
			div = 0.0005;
			break;
		case ANIM_PAT_SLOW :
			div = 0.001;
			break;
		case ANIM_PAT_MEDIUM :
			div = 0.002;
			break;
		case ANIM_PAT_FAST :
			div = 0.004;
			break;
		case ANIM_PAT_VERY_FAST :
			div = 0.008;
			break;
		}
		int s = (int)(Math.sqrt(dx*dx+dy*dy+dp*dp)/div);
		Double[] dv = new Double[]{(pt2[0]-pt1[0])/s,(pt2[1]-pt1[1])/s};
		Double[] tmp = pt1.clone();
		for(int k = 0;k<s;k++){
			dmath.add(tmp,dv,tmp);
			starts.add(tmp.clone());
		}
		Iterator<Double[]> it = starts.iterator();
		while(it.hasNext()&&fltCtr.getShapeLocal(it.next())!=null) 
			it.remove();
		List<Integer> isctIdxs = new ArrayList<Integer>();
		List<IShape<Double>> shapes = new ArrayList<IShape<Double>>();
		boolean in1 = false;
		for(int i = 0;i<starts.size();i++){
			Double[] pt = starts.get(i);
			Double[] ctr1 = in1?shape.getCenter().clone():null;
			shape = fltCtr.getShapeLocal(pt);
			boolean in2 = shape !=null;
			Double[] ctr2 = in2?shape.getCenter():null;
			if(in1&&in2&&(ctr1[0]!=ctr2[0]||ctr1[1]!=ctr2[1])){
				isctIdxs.add(i);
				isctIdxs.add(i-1);
				shapes.add(shape.copy());
			} else if(in1!=in2)
				if(in2){
					isctIdxs.add(i-1);
					shapes.add(shape.copy());
				} else
					isctIdxs.add(i);
			in1 = in2;
		}
		List<Double[]> starts2 = new ArrayList<Double[]>();
		int idxEnd = 0;
		for(int i = 0;i<isctIdxs.size();i+=2){
			int idx1 = isctIdxs.get(i);
			int idx2 = isctIdxs.get(i+1);
			shape = shapes.get(i/2);
			double r = shape.getSize().doubleValue();
			pt1 = starts.get(idx1);
			pt2 = starts.get(idx2);
			Double[] ctr = shape.getCenter();
			Double[] dn = new Double[]{pt2[1]-pt1[1],-(pt2[0]-pt1[0])};
			Double[] ptm = new Double[]{(pt2[0]+pt1[0])/2,(pt2[1]+pt1[1])/2};
			Double[] du = new Double[]{ctr[0]-ptm[0],ctr[1]-ptm[1]};
			double dbeta = dmath.norm(dv)/r;
			double betaMax = 2*Math.acos(dmath.norm(du)/r);
			betaMax = Math.abs(betaMax);
			if((ctr[0]-ptm[0])*dn[0]+(ctr[1]-ptm[1])*dn[1]>0)
				dbeta = -dbeta;
			tmp = pt1.clone();
			Double[] sinCos = dmath.sincos(dbeta);
			starts2.addAll(starts.subList(idxEnd,idx1+1));
			idxEnd = idx2;
			for(double beta = 0;Math.abs(beta)<betaMax;beta+=dbeta){
				dmath.rotate2D(tmp,ctr,sinCos[0],sinCos[1]);
				starts2.add(tmp.clone());
			}
		}
		starts2.addAll(starts.subList(idxEnd,starts.size()));
		fltCtr.rotate(dmath.ZERO2D,null,-dphi);
		double dalpha = dphi/starts2.size();
		ReflectionEngine<Double> fEng = new ReflectionEngine<Double>(dmath,4,2);
		fEng.setOutputType(ReflectionEngine.OUTPUT_LOCAL);
		RayPolygon<Double> fpoly = new RayPolygon<Double>(dmath.util,3);
		Double[] dir = new Double[]{-1.0,0.0};
		int rcnt = -1;
		for(int i = 0;i<starts2.size()-1;i++){
			fltCtr.rotate(dmath.ZERO2D,null,dalpha);
			fltCtr.local2WorldPoint(starts2.get(i),tmp);
			Ray<Double> testRay = new Ray<Double>(tmp,dir,dmath.util);
			fEng.calcStatic(testRay,fltCtr,fpoly,null);
			if(((rcnt>-1&&fpoly.size()>=1)||(rcnt>0))&&
				((rcnt<3&&fpoly.size()<4)||(rcnt<4&&fpoly.size()<3)) ){
				fltCtr.rotate(dmath.ZERO2D,null,-dalpha/2);
				Double[] pt = new Double[2];
				pt[0] = (starts2.get(i)[0]+starts2.get(i-1)[0])/2;
				pt[1] = (starts2.get(i)[1]+starts2.get(i-1)[1])/2;
				fltCtr.local2WorldPoint(pt.clone(),pt);
				T[] start = tmath.util.cast(pt);
				start[0] = tmath.util.copy(start[0],tmath.PRECISION);
				start[1] = tmath.util.copy(start[1],tmath.PRECISION);
				Ray<T> iRay = new Ray<T>(start,model.LIGHT_SOURCE_DIR,tmath.util);
				iRay.setAttribute(INTERPOLATE);
				rays.add(iRay);
				fltCtr.rotate(dmath.ZERO2D,null,dalpha/2);
			}
			rcnt = fpoly.size();
			T[] start = tmath.util.cast(tmp);
			start[0] = tmath.util.copy(start[0],tmath.PRECISION);
			start[1] = tmath.util.copy(start[1],tmath.PRECISION);
			rays.add(new Ray<T>(start,model.LIGHT_SOURCE_DIR,tmath.util));
		}
		rays.add(new Ray<T>(pos,model.LIGHT_SOURCE_DIR,tmath.util));
	}
}