/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JSeparator;
import javax.swing.filechooser.FileNameExtensionFilter;

import lattice.Ray;
import math.VecMath;
import mvc.mgr.LightPatternManager;
import pattern.BaseLightPattern;
import pattern.CalculationRecords;
import pattern.LightChar;
import pattern.LightCharPattern;
import calc.CalculatorView;
import calc.CalculatorVisualizer;

import common.ColorTheme;
import common.FileIO;
import common.Globals;
import common.Log;
import common.SelectTextItemListener;

@SuppressWarnings("serial")
public class MainMenuBar<T extends Number> extends JMenuBar {
	
	static final String ANIM_PAT_ENTRY_SLOW = "Slow";
	static final String ANIM_PAT_ENTRY_MEDIUM = "Medium";
	static final String ANIM_PAT_ENTRY_FAST = "Fast";
	static final String ANIM_PAT_ENTRY_VERY_FAST = "Turbo";
	static final String SYMBOLS_ENTRY_BORDER = "Border...";
	private final View<T> view;
	private final ColorTheme[] themes;
	private boolean allowEvents;
	private final ArrayList<AbstractButton> animGroup;
	private CalculatorView<T> calcView;
	private Font smFont;
	private int threshold;
	JMenu precMenu;
	JMenu gridSizeMenu;
	JMenu shapeSizeMenu;
	JMenu animMenu;
	JMenu patternMenu;
	JMenu msgMenu;
	JMenu settingsMenu;
	JMenu mapMenu;
	JMenu themeMenu;
	private JMenuItem defItem;
	private JCheckBoxMenuItem sortItem; 
	private final Color PINK;
	public MainMenuBar(View<T> vw,ColorTheme theme) {
		view = vw;
		allowEvents = true;
		animGroup = new ArrayList<AbstractButton>();
		List<ColorTheme> cts = new FileIO<T>(null).loadColorThemes();
		cts.add(0,new ColorTheme());
		themes = cts.toArray(new ColorTheme[cts.size()]);
		Font f = getFont();
		smFont = new Font(f.getFontName(),f.getStyle(),f.getSize()-1);
		threshold = 2;
		Color p = Color.pink;
		PINK = new Color(p.getRed(),p.getGreen()+20,p.getBlue()+20);
		initGUI(theme);
		Model<T> model = view.getController().getModel();
		model.addModelListener(new IModelListener() {

			@Override
			public void modelChanged(byte event) {
				switch(event) {
				case Model.SHAPE_RESIZE:
					updateShapeSizeMenu();
					break;
				case Model.LIGHTPATTERNS_UPDATE:
					rebuildPatternMenu();
					break;
				case Model.MODEL_NEW:
					if(calcView!=null)
						calcView.getVisualizer().getModel().clearResults();
					updateShapeSizeMenu();
				}
			}
			private void updateShapeSizeMenu() {
				Model<T> ml = view.getController().getModel();
				byte selIdx = ml.getShapeSizeIdx();
				setSelectedIdx(selIdx,shapeSizeMenu,false);
				Globals.IDX_SHAPE_SIZE = selIdx;
			}
		});
	}
	private void initGUI(ColorTheme theme) {
		VecMath<T> math = view.getController().getModel().getMath();
		final FileIO<T> fileIO = new FileIO<T>(math);
		patternMenu = new JMenu("Patterns");
		defItem = new JMenuItem("Show Defining Path");
		configure(defItem);
		defItem.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseReleased(MouseEvent me) {
				Controller<T> ctrl = view.getController();
				if(ctrl.isRotAnimRun())
					return;
				else if(ctrl.isPatAnimRun())
					return;
				ctrl.waitForCalculatorRelease();
				LightPatternSelector<T> sel = 
						view.getLightPatternSelector();
				BaseLightPattern<T> blp = sel.getSelected();
				if(blp!=null){
					view.getReflectionVisualizer().requestDefPath(blp);
					view.getReflectionVisualizer().repaint();
				}	
			}
		});
		rebuildPatternMenu();
		add(patternMenu);
		settingsMenu = new JMenu("Settings");
		gridSizeMenu = new JMenu("Grid Size");
		configure(gridSizeMenu);
		rebuildGridSizeMenu((short)-1,false);
		shapeSizeMenu = new JMenu("Radius");
		configure(shapeSizeMenu);
		ItemListener il = new ItemListener() {

			@Override
			public void itemStateChanged(ItemEvent e) {
				if(allowEvents&&e.getStateChange()==ItemEvent.SELECTED) {
					for(byte i = 0;i<shapeSizeMenu.getItemCount();i++)
						if(shapeSizeMenu.getItem(i)!=e.getSource())
							shapeSizeMenu.getItem(i).setSelected(false);
					String n = ((JMenuItem)e.getSource()).getText();
					String num = n.substring(0,n.indexOf("%")).trim();
					byte b = Byte.parseByte(num);
					Controller<T> ctrl = view.getController(); 
					float speed = 0;
					byte speed2 = AnimatePatterns.ANIM_PAT_MEDIUM;
					int pos = -1;
					if(ctrl.isRotAnimRun()){
						speed = ctrl.getRotAnimSpeed();
						ctrl.stopRotAnim();
					} else if(ctrl.isPatAnimRun()){
						pos = ctrl.getPatAnimPos();
						speed2 = ctrl.getPatAnimSpeed();
						ctrl.stopPatAnim();
					}
					ctrl.stopCalculation();
					ctrl.waitForCalculatorRelease();
					ctrl.getModel().setShapePercentSize(b);
					Ray<T> ray = ctrl.getModel().getRay();
					ctrl.getModel().setRay(ray,Model.STATIC_CALC);
					if(speed!=0)
						ctrl.startRotAnim(speed);
					else if(pos!=-1)
						ctrl.startPatAnim(pos,(short)-1,speed2);
					else
						ctrl.startCalculation();
				}
			}
		};
		for(byte i = 0;i<Globals.SHAPE_SIZES.length;i++) {
			short s = Globals.SHAPE_SIZES[i];
			JMenuItem rbmi = new JRadioButtonMenuItem(s+" %");
			configure(rbmi);
			if(i==Globals.IDX_SHAPE_SIZE)
				rbmi.setSelected(true);
			rbmi.addItemListener(il);
			shapeSizeMenu.add(rbmi);
		}
		il = new ItemListener() {

			@Override
			public void itemStateChanged(ItemEvent e) {
				if(allowEvents&&e.getStateChange()==ItemEvent.SELECTED) {
					String txt = ((JMenuItem)e.getSource()).getText();
					short p = Short.parseShort(txt.trim());
					Controller<T> ctrl = view.getController();
					float speed = 0;
					byte speed2 = AnimatePatterns.ANIM_PAT_MEDIUM;
					int pos = -1;
					if(ctrl.isRotAnimRun()){
						speed = ctrl.getRotAnimSpeed();
						ctrl.stopRotAnim();
					} else if(ctrl.isPatAnimRun()){
						pos = ctrl.getPatAnimPos();
						speed2 = ctrl.getPatAnimSpeed();
						ctrl.stopPatAnim();
					}
					ctrl.stopCalculation();
					ctrl.stopScan("Stopped on precision change");
					ctrl.waitForCalculatorRelease();
					view.getController().setPrecision(p);
					if(speed!=0)
						ctrl.startRotAnim(speed);
					else if(pos!=-1)
						ctrl.startPatAnim(pos,(short) -1,speed2);
					else
						ctrl.startCalculation();
					Globals.IDX_PREC = 
						Globals.getIndex(p,Globals.PRECISIONS);
				}
			}
		};
		ButtonGroup bg = new ButtonGroup();
		precMenu = new JMenu("Precision");
		configure(precMenu);
		for(short i = 0;i<Globals.PRECISIONS.length;i++) {
			short p = Globals.PRECISIONS[i];
			JMenuItem rbmi = new JRadioButtonMenuItem(p+"");
			configure(rbmi);
			if(i==Globals.IDX_PREC)
				rbmi.setSelected(true);
			rbmi.addItemListener(il);
			precMenu.add(rbmi);
			bg.add(rbmi);
		}
		JCheckBoxMenuItem autoPItem = 
				new JCheckBoxMenuItem("Automatic Precision");
		configure(autoPItem);
		autoPItem.addItemListener(new ItemListener() { 
				@Override
				public void itemStateChanged(ItemEvent ie) {
					Globals.AUTO_PREC = 
						ie.getStateChange()==ItemEvent.SELECTED;
				}
			});
		autoPItem.setSelected(true);
		JCheckBoxMenuItem autoRItem = 
				new JCheckBoxMenuItem("Automatic Radius");
		configure(autoRItem);
		autoRItem.addItemListener(new ItemListener() { 
				@Override
				public void itemStateChanged(ItemEvent ie) {
					Globals.AUTO_RADIUS = 
						ie.getStateChange()==ItemEvent.SELECTED;
				}
			});
		autoRItem.setSelected(true);
		JCheckBoxMenuItem muItem = 
				new JCheckBoxMenuItem("Museum");
		configure(muItem);
		muItem.addItemListener(new ItemListener() { 
				@Override
				public void itemStateChanged(ItemEvent ie) {
					view.setMuseum();
				}
			});
		JCheckBoxMenuItem fsItem = 
				new JCheckBoxMenuItem("Museum (Fullscreen)");
		configure(fsItem);
		fsItem.addItemListener(new ItemListener() { 
				@Override
				public void itemStateChanged(ItemEvent ie) {
					view.setMuseum();
					view.setFullScreen();
				}
			});
		themeMenu = new JMenu("Theme");
		configure(themeMenu);
		bg = new ButtonGroup();
		for(ColorTheme ct:themes) {
			JRadioButtonMenuItem item = 
				new JRadioButtonMenuItem(ct.getName());
			configure(item);
			if(ct.getName().compareTo(theme.getName())==0)
				item.setSelected(true);
			item.addItemListener(new ThemeListener(ct));
			themeMenu.add(item);
			bg.add(item);
		}
		JMenu lightSourceMenu = new JMenu("Light Source");
		configure(lightSourceMenu);
		String[] lightSources = new String[]{"Laser 1",
											 "Laser 2",
											 "Laser 3",
											 "Invisible"}; 
		bg = new ButtonGroup();
		for(int i = 0;i<lightSources.length;i++){		
			JCheckBoxMenuItem lsItem = createCheckBox(lightSources[i]);
			if(i == 0)
				lsItem.setSelected(true);
			final int j = i;
			lsItem.addItemListener(new ItemListener() { 
				@Override
				public void itemStateChanged(ItemEvent ie) {
					if(ie.getStateChange()==ItemEvent.SELECTED)
						view.setLightSource(Visualizer.LIGHT_SOURCES[j]);
				}
			});
			lightSourceMenu.add(lsItem);
			bg.add(lsItem);
		}
		final JCheckBoxMenuItem varItem = createCheckBox("Variables...");
		varItem.addItemListener(new ItemListener() {

			JFrame vf;
			@Override
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED) {
					vf = view.createVariableView();
					vf.addWindowListener(new WindowAdapter() {

						@Override
						public void windowClosed(WindowEvent we) {
							varItem.setSelected(false);
							vf = null;
						}
						@Override
						public void windowClosing(WindowEvent we) {
							varItem.setSelected(false);
							vf = null;
						}
					});
				} else if(vf!=null)
					vf.dispose();
			}
		});
		sortItem = createCheckBox("Size Sort Patterns");
		sortItem.setSelected(true);
		sortItem.addItemListener(new ItemListener() {

			@Override
			public void itemStateChanged(ItemEvent ie) {
				rebuildPatternMenu();
			}
		});
		JMenu thresholdMenu = new JMenu("Group Patterns");
		configure(thresholdMenu);
		bg = new ButtonGroup();
		for(int i = 1;i < 6;i++){
			JCheckBoxMenuItem treshItem = createCheckBox(i+"");
			if(i-1 == threshold)
				treshItem.setSelected(true);
			treshItem.addItemListener(new ItemListener() { 
				@Override
				public void itemStateChanged(ItemEvent ie) {
					if(ie.getStateChange()==ItemEvent.SELECTED){
						String txt = ((JMenuItem)ie.getSource()).getText();
						threshold = Integer.parseInt(txt)-1;
						rebuildPatternMenu();
					}
				}
			});
			thresholdMenu.add(treshItem);
			bg.add(treshItem);
		}
		settingsMenu.add(gridSizeMenu);
		settingsMenu.add(shapeSizeMenu);
		settingsMenu.add(precMenu);
		settingsMenu.add(autoPItem);
		settingsMenu.add(autoRItem);
		settingsMenu.add(varItem);
		settingsMenu.add(new JSeparator());
		settingsMenu.add(themeMenu);
		settingsMenu.add(lightSourceMenu);
		settingsMenu.add(new JSeparator());
		settingsMenu.add(sortItem);
		settingsMenu.add(thresholdMenu);
		add(settingsMenu);
		JMenu utilMenu = new JMenu("Util");
		final JCheckBoxMenuItem calcItem = createCheckBox("Calculator...");
		calcItem.addItemListener(new CalculatorListener(calcItem));
		mapMenu = new JMenu("Symbols of Light");
		configure(mapMenu);
		final JCheckBoxMenuItem borderMapItem = createCheckBox("Border...");
		borderMapItem.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent ie) {
				if(ie.getStateChange()==ItemEvent.SELECTED){
					Controller<T> ctrl = view.getController();
					LightChar[] lcs = ctrl.getModel().getLightChars();
					VecMath<T> mt = ctrl.getModel().getMath();
					VecMath<Float> fmt = ctrl.getModel().getFloatMath();
					IFZLBorder b1 = new LightCharBorder<T>(lcs,mt,fmt);
					ColorTheme th = view.getReflectionVisualizer().getTheme();
					IFZLBorder b2 = new SimpleBorder(5,th.getBorderColor());
					IFZLBorder b3 = new CompoundBorder(b1,b2);
					view.getReflectionVisualizer().setBorder(b3);
				} else {
					ColorTheme th = view.getReflectionVisualizer().getTheme();
					IFZLBorder b = new SimpleBorder(5,th.getBorderColor());
					view.getReflectionVisualizer().setBorder(b);
				}
			}
		});
		final JCheckBoxMenuItem tableMapItem = createCheckBox("Table...");
		tableMapItem.addItemListener(new ItemListener() {
			JFrame frame;
			@Override
			public void itemStateChanged(ItemEvent ie) {
				if(ie.getStateChange()==ItemEvent.SELECTED) {
					frame = new JFrame("FroZenLight - Characters of Light");
					Model<T> model = view.getController().getModel();
					LightChar[] lc = model.getLightChars();
					Controller<T> ctrl = view.getController();
					VecMath<T> mt = ctrl.getModel().getMath();
					VecMath<Float> fmt = ctrl.getModel().getFloatMath();
					final LightCharTable<T> lcm = 
						new LightCharTable<T>(lc,mt,fmt);
					frame.add(lcm);
					frame.pack();
					frame.setSize(500,500);
					frame.setVisible(true);
					frame.addWindowListener(new WindowAdapter() {
						@Override
						public void windowClosed(WindowEvent e) {
							tableMapItem.setSelected(false);
							lcm.dispose();
						}
						@Override
						public void windowClosing(WindowEvent e) {
							tableMapItem.setSelected(false);
							lcm.dispose();
						}
					});
				} else {
					if(frame!=null)
						frame.dispose();
					frame = null;
				}
			}
		});
		mapMenu.add(borderMapItem);
		mapMenu.add(tableMapItem);
		final JCheckBoxMenuItem managerItem = createCheckBox("Pattern Manager...");
		managerItem.addItemListener(new ItemListener() {

			JFrame frame;
			@Override
			public void itemStateChanged(ItemEvent ie) {
				if(ie.getStateChange()==ItemEvent.SELECTED) {
					frame = new LightPatternManager<T>(view.getController());
					frame.pack();
					frame.setSize(600,400);
					frame.setVisible(true);
					frame.addWindowListener(new WindowAdapter() {

						@Override
						public void windowClosed(WindowEvent e) {
							managerItem.setSelected(false);
						}
						@Override
						public void windowClosing(WindowEvent e) {
							managerItem.setSelected(false);
						}
					});
				} else {
					if(frame!=null)
						frame.dispose();
					frame = null;
				}
			}
		});
		final JCheckBoxMenuItem scannerItem = createCheckBox("Scanner...");
		scannerItem.addItemListener(new ItemListener() {

			JFrame vf;
			@Override
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED) {
					vf = view.createScanView();
					vf.addWindowListener(new WindowAdapter() {

						@Override
						public void windowClosed(WindowEvent we) {
							scannerItem.setSelected(false);
							vf = null;
						}
						@Override
						public void windowClosing(WindowEvent we) {
							scannerItem.setSelected(false);
							vf = null;
						}
					});
				} else if(vf!=null)
					vf.dispose();
			}
		});
		final JCheckBoxMenuItem logItem = createCheckBox("Logs...");
		logItem.addItemListener(new ItemListener() {

			JFrame vf;
			@Override
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED) {
					vf = new JFrame("Logs");
					Log.getIstc().initGUI();
					vf.getContentPane().add(Log.getIstc());
					vf.setSize(400,300);
					vf.setVisible(true);
					vf.addWindowListener(new WindowAdapter() {

						@Override
						public void windowClosed(WindowEvent we) {
							logItem.setSelected(false);
							if(vf!=null)
								vf.getContentPane().remove(0);
							vf = null;
						}
						@Override
						public void windowClosing(WindowEvent we) {
							logItem.setSelected(false);
							if(vf!=null)
								vf.getContentPane().remove(0);
							vf = null;
						}
					});
					vf.repaint();
				} else if(vf!=null){
					vf.dispose();
					Log.getIstc().deleteGUI();
				}
			}
		});
		JMenuItem exitMenu = new JMenuItem("Exit");	
		exitMenu.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseReleased(MouseEvent me) {
				System.exit(0);
			}
		});
		JMenu recordItem = new JMenu("Record");
		configure(recordItem);
		JMenuItem printItem = new JMenuItem("Print");
		configure(printItem);
		printItem.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseReleased(MouseEvent me) {
				Controller<T> ctrl = view.getController();
				if(ctrl.isRotAnimRun())
					ctrl.stopRotAnim();
				else if(ctrl.isPatAnimRun())
					ctrl.stopPatAnim();
				ctrl.waitForCalculatorRelease();
				Model<T> model = view.getController().getModel();
				String txt = model.createRecord("",-1).toString();
				Log.getIstc().log(txt);
			}
		});
		JMenuItem saveItem = new JMenuItem("Save As...");
		configure(saveItem);
		saveItem.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseReleased(MouseEvent me) {
				Controller<T> ctrl = view.getController();
				if(ctrl.isRotAnimRun())
					ctrl.stopRotAnim();
				else if(ctrl.isPatAnimRun())
					ctrl.stopPatAnim();
				JFileChooser fc = new JFileChooser();
			    FileNameExtensionFilter filter = 
			    	new FileNameExtensionFilter("Text File (*.txt)","txt");
			    fc.setFileFilter(filter);
			    fc.setDialogTitle("Save Record");
			    fc.setSelectedFile(new File("record.txt"));
			    int retVal = fc.showOpenDialog(getParent());
			    if(retVal == JFileChooser.APPROVE_OPTION) {
			    	ctrl.waitForCalculatorRelease();
					Model<T> model = view.getController().getModel();
					CalculationRecords<T> recs = 
						model.createRecord("",-1);
					File file = fc.getSelectedFile();
					if(file!=null)
						model.addRecords(recs,file);
			    }
			}
		});
		JMenuItem loadItem = new JMenuItem("Load...");
		configure(loadItem);		
		loadItem.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseReleased(MouseEvent me) {
				Controller<T> ctrl = view.getController();
				if(ctrl.isRotAnimRun())
					ctrl.stopRotAnim();
				else if(ctrl.isPatAnimRun())
					ctrl.stopPatAnim();
				JFileChooser fc = new JFileChooser("record.txt");
			    FileNameExtensionFilter filter = 
			    	new FileNameExtensionFilter("Text file (*.txt)","txt");
			    fc.setDialogTitle("Load Record");
			    fc.setFileFilter(filter);
			    int retVal = fc.showOpenDialog(getParent());
			    if(retVal!=JFileChooser.APPROVE_OPTION) 
			    	return;
				File file = fc.getSelectedFile();
				if(file==null)
					return;
				CalculationRecords<T> recs = 
					ctrl.getModel().getRecords(file);
				ctrl.waitForCalculatorRelease();
				if(recs!=null){
					CalculationRecords<T>.Record rec = 
						recs.getRecord(0);
					VecMath<T> mt = ctrl.getModel().getMath();
					T h = mt.util.parse(rec.getHeight().toString());
					int p = mt.util.precision(h);
					if(Globals.adjustPrecision(p))
						setSelectedIdx(Globals.IDX_PREC,precMenu,true);
				}
				Model<T> model = view.getController().getModel();
				model.setRecord(recs.getRecord(0),recs.getGridSize());
				ctrl.startCalculation();
			}
		});
		recordItem.add(printItem);
		recordItem.add(saveItem);
		recordItem.add(loadItem);
		utilMenu.add(mapMenu);
		utilMenu.add(new JSeparator());
		utilMenu.add(managerItem);
		utilMenu.add(calcItem);
		utilMenu.add(scannerItem);
		utilMenu.add(logItem);
		utilMenu.add(recordItem);
		utilMenu.add(new JSeparator());
		utilMenu.add(fsItem);
		utilMenu.add(muItem);
		utilMenu.add(new JSeparator());
		utilMenu.add(exitMenu);
		add(utilMenu);
		animMenu = new JMenu("Animation");
		JMenuItem rotHeader = new JMenuItem("Rotation");
		configureHeader(rotHeader);
		JMenuItem piItem = new JRadioButtonMenuItem("1   pi/min");
		configure(piItem);
		piItem.addItemListener(new AnimRotListener(Math.PI));
		JMenuItem piDiv2Item = new JRadioButtonMenuItem("1/2 pi/min");
		configure(piDiv2Item);
		piDiv2Item.addItemListener(new AnimRotListener(Math.PI/2));
		JMenuItem piDiv4Item = new JRadioButtonMenuItem("1/4 pi/min");
		configure(piDiv4Item);
		piDiv4Item.addItemListener(new AnimRotListener(Math.PI/4));
		JMenuItem piDiv8Item = new JRadioButtonMenuItem("1/8 pi/min");
		configure(piDiv8Item);
		piDiv8Item.addItemListener(new AnimRotListener(Math.PI/8));
		JMenuItem patHeader = new JMenuItem("Symmetric Patterns");
		configureHeader(patHeader);
		JMenuItem patterns1 = new JRadioButtonMenuItem(ANIM_PAT_ENTRY_SLOW);	
		configure(patterns1);
		patterns1.addItemListener(new AnimPatListener(AnimatePatterns.ANIM_PAT_SLOW));
		JMenuItem patterns2 = new JRadioButtonMenuItem(ANIM_PAT_ENTRY_MEDIUM);	
		configure(patterns2);
		patterns2.addItemListener(new AnimPatListener(AnimatePatterns.ANIM_PAT_MEDIUM));
		JMenuItem patterns3 = new JRadioButtonMenuItem(ANIM_PAT_ENTRY_FAST);	
		configure(patterns3);
		patterns3.addItemListener(new AnimPatListener(AnimatePatterns.ANIM_PAT_FAST));
		JMenuItem patterns4 = new JRadioButtonMenuItem(ANIM_PAT_ENTRY_VERY_FAST);	
		configure(patterns4);
		patterns4.addItemListener(new AnimPatListener(AnimatePatterns.ANIM_PAT_VERY_FAST));
		animMenu.add(rotHeader);
		animMenu.add(piItem);
		animMenu.add(piDiv2Item);
		animMenu.add(piDiv4Item);
		animMenu.add(piDiv8Item);
		animMenu.add(patHeader);
		animMenu.add(patterns1);
		animMenu.add(patterns2);
		animMenu.add(patterns3);
		animMenu.add(patterns4);
		animGroup.add(piItem);
		animGroup.add(piDiv2Item);
		animGroup.add(piDiv4Item);
		animGroup.add(piDiv8Item);
		animGroup.add(patterns1);
		animGroup.add(patterns2);
		animGroup.add(patterns3);
		animGroup.add(patterns4);
		add(animMenu);
		JMenu infoMenu = new JMenu("?");
		final JCheckBoxMenuItem helpItem = createCheckBox("Help...");
		String text = fileIO.loadInfoText("Main");
		il = new SelectTextItemListener(helpItem,"Help",text,570,600);
		helpItem.addItemListener(il);
		final JCheckBoxMenuItem aboutItem = createCheckBox("About...");
		text = fileIO.loadInfoText("About");
		il = new SelectTextItemListener(aboutItem,"About",text,350,300);
		aboutItem.addItemListener(il);
		infoMenu.add(helpItem);
		infoMenu.add(aboutItem);
		add(infoMenu);
	}
	void rebuildGridSizeMenu(short grs,boolean fireEvent) {
		allowEvents = fireEvent;
		int l = Globals.GRID_SIZES.length;
		Globals.addGridSize(grs);
		if(l<Globals.GRID_SIZES.length)
			Globals.IDX_GRID_SIZE = Globals.getIndex(grs,Globals.GRID_SIZES);
		gridSizeMenu.removeAll();
		ItemListener il = new ItemListener() {

			@Override
			public void itemStateChanged(ItemEvent e) {
				if(allowEvents&&e.getStateChange()==ItemEvent.SELECTED) {
					String n = ((JMenuItem)e.getSource()).getText();
					short s = Short.parseShort(n.substring(0,n.indexOf("x")));
					Controller<T> ctrl = view.getController();
					if(ctrl.getModel().getGridSize()!=s) {
						float speed1 = 0;
						byte speed2 = AnimatePatterns.ANIM_PAT_MEDIUM;
						int pos = -1;
						if(ctrl.isRotAnimRun()){
							speed1 = ctrl.getRotAnimSpeed();
							ctrl.stopRotAnim();
						} else if(ctrl.isPatAnimRun()){
							pos = ctrl.getPatAnimPos();
							speed2 = ctrl.getPatAnimSpeed();
							ctrl.stopPatAnim();
						}
						ctrl.stopCalculation();
						ctrl.waitForCalculatorRelease();
						Globals.IDX_GRID_SIZE = Globals.getIndex(s,Globals.GRID_SIZES);
						ctrl.getModel().setGridSize(s);
						Ray<T> ray = view.getController().getModel().getRay();
						ctrl.getModel().setRay(ray,Model.STATIC_CALC);
						if(speed1!=0)
							ctrl.startRotAnim(speed1);
						else if(pos!=-1)
							ctrl.startPatAnim(-1,s,speed2);
						else
							ctrl.startCalculation();
					}
				}
			}
		};
		ButtonGroup bg = new ButtonGroup();
		for(short i = 0;i<Globals.GRID_SIZES.length;i++) {
			short gs = Globals.GRID_SIZES[i];
			JMenuItem rbmi = new JRadioButtonMenuItem(gs+"x"+gs);
			configure(rbmi);
			rbmi.addItemListener(il);
			if(i==Globals.IDX_GRID_SIZE)
				rbmi.setSelected(true);
			gridSizeMenu.add(rbmi);
			bg.add(rbmi);
		}
		JMenuItem rbmi = new JMenuItem("other...");
		configure(rbmi);
		ActionListener al = new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				if(allowEvents) {
					String txt = "Enter the desired grid size:";
					String value = JOptionPane.showInputDialog(view,txt);
					short size;
					try {
						int idx = value.indexOf("x");
						if(idx!=-1)
							value = value.substring(0,idx);
						size = Short.parseShort(value);
					} catch(NumberFormatException nfe) {
						return;
					}
					rebuildGridSizeMenu(size,true);
				}
			}
		};
		rbmi.addActionListener(al);
		gridSizeMenu.add(rbmi);
		allowEvents = true;
	}
	private void rebuildPatternMenu() {
		patternMenu.removeAll();
		final Controller<T> ctrl = view.getController();
		Model<T> model = ctrl.getModel();
		List<BaseLightPattern<T>> patterns = model.getLightPatterns();
		final JMenuItem menuItem = new JMenuItem("Symmetric");
		configureHeader(menuItem);
		patternMenu.add(menuItem);
		Iterator<BaseLightPattern<T>> iter = patterns.iterator();
		while(iter.hasNext())
			if(iter.next() instanceof LightCharPattern)
				iter.remove();
		cluster(patterns);
		patterns = model.getLightPatterns();
		final JMenuItem menuItem2 = new JMenuItem("Symbol Sequence");
		configureHeader(menuItem2);
		patternMenu.add(menuItem2);
		iter = patterns.iterator();
		while(iter.hasNext())
			if(!(iter.next() instanceof LightCharPattern))
				iter.remove();
		cluster(patterns);
		patternMenu.add(new JSeparator());
		patternMenu.add(defItem);
	}
	private void cluster(List<BaseLightPattern<T>> patterns){
		ArrayList<String[]> classes = new ArrayList<String[]>();
		boolean sizeSort = sortItem!=null?sortItem.isSelected():true;
		if(!sizeSort) {
			char[] az = "ABCDEFGHIJKLMNOPQRSTUVWXYZ".toCharArray();
			for(int i = 0;i<az.length;i+=2)
				classes.add(new String[]{az[i]+"",az[i+1]+""});
		} else {
			HashSet<Integer> sizesSet = new HashSet<Integer>();
			for(BaseLightPattern<T> p:patterns)
				sizesSet.add((int)p.getGridSize());
			ArrayList<Integer> sortedSizes = new ArrayList<Integer>(sizesSet);
			Collections.sort(sortedSizes);
			for(int i = 0;i<sortedSizes.size();i++)
				classes.add(new String[]{sortedSizes.get(i)+""});
		}
		if(patterns==null)
			patterns = new ArrayList<BaseLightPattern<T>>();
		for(int i = 0;i<classes.size();i++) {
			String start = classes.get(i)[0];
			String end = classes.get(i)[classes.get(i).length>1?1:0];
			List<BaseLightPattern<T>> tmpList = new ArrayList<BaseLightPattern<T>>();
			for(BaseLightPattern<T> p:patterns)
				if(sizeSort) {
					if(p.getGridSize()==Short.parseShort(end))
						sortedInsert(p,tmpList);
				} else {
					String n = p.getName();
					if(n.compareToIgnoreCase(start)>=0)
						if(n.compareToIgnoreCase(end)<0||n.startsWith(end))
							sortedInsert(p,tmpList);
				}
			if(tmpList.size()>0) {
				JMenu menu = null;
				if(tmpList.size()>threshold){
					String text;
					if(sizeSort)
						text = start;
					else
						text = start+" - "+end;
					menu = new JMenu(text);
					menu.setFont(smFont);
					configure(menu);
					patternMenu.add(menu);
				}
				for(int j = 0;j<tmpList.size();j++) {
					final BaseLightPattern<T> lp = tmpList.get(j);
					String name = lp.getName();
					if(sizeSort&&tmpList.size()<threshold+1){
						if(lp.getGridSize()>9)
							name = lp.getGridSize()+": "+name;
						else
							name = lp.getGridSize()+":  "+name;
					}
					JMenuItem menuItem = new JMenuItem(name);
					configure(menuItem);
					MouseListener l = new MouseAdapter() {

						@Override
						public void mouseReleased(MouseEvent me) {
							LightPatternSelector<T> sel = 
									view.getLightPatternSelector();
							sel.select(null,lp,lp.isStoreCalcEnabled());
						}
					};
					menuItem.addMouseListener(l);
					if(tmpList.size()>threshold)
						menu.add(menuItem);
					else
						patternMenu.add(menuItem);
				}
			}
		}		
	}
	// Workaround to fix displacement of menu items under check box items. 
	private JCheckBoxMenuItem createCheckBox(String txt){
		JCheckBoxMenuItem mi = new JCheckBoxMenuItem(txt){
			public Dimension getPreferredSize(){
				Dimension d = super.getPreferredSize();
				d.height -= 3;
				return d;
			}
		};
		mi.setFont(smFont);
		return mi;
	}
	private void configureHeader(JMenuItem mi){
		configure(mi);
		mi.setBackground(new Color(165,165,165));
		mi.setBorder(BorderFactory.createLineBorder(Color.black));
	}
	private void configure(JMenuItem mi){
		mi.setFont(smFont);
		int h = mi.getPreferredSize().height-3;
		int w = mi.getPreferredSize().width+10;
		mi.setPreferredSize(new Dimension(w,h));
		if(mi instanceof JMenu)
			mi.setForeground(PINK);
	}
	private void sortedInsert(BaseLightPattern<T> p,
								List<BaseLightPattern<T>> list) {
		int k = 0;
		while(k<list.size()) {
			if(list.get(k).getName().compareTo(p.getName())>0)
				break;
			k++;
		}
		list.add(k,p);
	}
	void deselectAnimationButtonGroup(AbstractButton skip) {
		for(AbstractButton b:animGroup)
			if(skip!=b&&b.isSelected())
				b.setSelected(false);
	}
	void setSelectedIdx(short idx,JMenu menu,boolean fireEvent) {
		allowEvents = fireEvent;
		for(byte i = 0;i<menu.getItemCount();i++)
			if(menu.getItem(i)!=null&&menu.getItem(i).isSelected())
				menu.getItem(i).setSelected(false);
		if(idx!=-1)
			((JRadioButtonMenuItem)menu.getItem(idx)).setSelected(true);
		allowEvents = true;
	}
	public void setSelected(String entry,JMenu menu,boolean fireEvent) {
		allowEvents = fireEvent;
		boolean entryFound = false;
		for(byte i = 0;i<menu.getItemCount();i++){
			JMenuItem it = menu.getItem(i);
			if(it!=null){
				if(it instanceof JMenu)
					setSelected(entry,(JMenu)it,fireEvent);
				else if(it.getText().indexOf(entry)>=0){
					entryFound = true;
					break;
				}
			}
		}
		if(entryFound)
			for(byte i = 0;i<menu.getItemCount();i++){
				JMenuItem it = menu.getItem(i);
				if(it!=null){
					if(it.getText().indexOf(entry)>=0){
						if(it instanceof JRadioButtonMenuItem)
							it.setSelected(true);
						else {
							MouseListener[] mls = it.getMouseListeners();
							mls[mls.length-1].mouseReleased(null);
						}
					} else if(it.isSelected())	
						it.setSelected(false);
				}
			}
		allowEvents = true;		
	}
	private final class AnimRotListener implements ItemListener {

		private final double phi;
		public AnimRotListener(double phi) {
			this.phi = phi;
		}
		public void itemStateChanged(ItemEvent ie) {
			if(ie.getStateChange()==ItemEvent.SELECTED) {
				deselectAnimationButtonGroup((AbstractButton)ie.getSource());
				view.getController().startRotAnim(phi);
			} else
				view.getController().stopRotAnim();
		}
	}
	private final class ThemeListener implements ItemListener {

		private final ColorTheme theme;
		public ThemeListener(ColorTheme t) {
			this.theme = t;
		}
		public void itemStateChanged(ItemEvent ie) {
			if(ie.getStateChange()==ItemEvent.SELECTED) {
				view.getReflectionVisualizer().setTheme(theme);
				view.getReflectionVisualizer().repaint();
			}
		}
	}
	private final class CalculatorListener implements ItemListener {

		private final JMenuItem item;
		CalculatorListener(JMenuItem item) {
			this.item = item;
		}
		@Override
		public void itemStateChanged(ItemEvent e) {
			if(e.getStateChange()==ItemEvent.SELECTED) {
				Model<T> model = view.getController().getModel();
				VecMath<T> math = model.getMath();
				CalculatorVisualizer<T> cvis = 
					new CalculatorVisualizer<T>(math);
				calcView = new CalculatorView<T>(cvis);
				calcView.addWindowListener(new WindowAdapter() {

					@Override
					public void windowClosed(WindowEvent we) {
						item.setSelected(false);
						calcView.removeWindowListener(this);
						calcView = null;
					}
					@Override
					public void windowClosing(WindowEvent we) {
						item.setSelected(false);
						calcView.removeWindowListener(this);
						calcView = null;
					}
				});
			} else
				calcView.dispose();
		}
	}
	private final class AnimPatListener implements ItemListener {
	
		private byte speed;
		AnimPatListener(byte speeeed){
			speed = speeeed;
		}
		public void itemStateChanged(ItemEvent ie) {
			if(ie.getStateChange()==ItemEvent.SELECTED) {
				deselectAnimationButtonGroup((AbstractButton)ie.getSource());
				view.getController().startPatAnim(-1,(short) -1,speed);
			} else
				view.getController().stopPatAnim();
		}
	}
}
