/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc.mgr;
import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.Border;

import math.VecMath;
import mvc.Controller;
import mvc.Model;
import pattern.BaseLightPattern;
import pattern.ILightPattern;

import common.Globals;

@SuppressWarnings("serial")
public abstract class BaseLightPatternEditor<T extends Number,P extends BaseLightPattern<T>>
        extends JPanel {

	private JTextField nameField;
	private JTextField sizeField;
	private JTextField maxRField;
	private JTextField precField;
	private JTextField timeField;
	protected final VecMath<T> math;
	protected final int columns;
	private JButton estimateButton;
	private Estimator estimator;
	public BaseLightPatternEditor(boolean editable,int cols,VecMath<T> mt) {
		this.columns = cols;
		this.math = mt;
		initGui(editable);
	}
	void initGui(boolean editable) {
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		setLayout(gridbag);
		int row = 0;
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0;
		c.gridy = row;
		c.weightx = 1;
		c.weighty = 0;
		c.gridwidth = columns;
		c.gridheight = 1;
		Border tb = BorderFactory.createTitledBorder("Name");
		nameField = new JTextField();
		nameField.setBackground(Globals.INPUT_BG);
		JPanel l = new JPanel(new GridLayout());
		l.add(nameField);
		l.setBorder(tb);
		gridbag.setConstraints(l,c);
		add(l);
		row = initGui(gridbag,c,row+1);
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0;
		c.gridy = row;
		c.weightx = 1;
		c.weighty = 0;
		c.gridwidth = columns;
		c.gridheight = 1;
		GridBagLayout gridbag2 = new GridBagLayout();
		GridBagConstraints c2 = new GridBagConstraints();
		JPanel p = new JPanel(gridbag2);
		c2.fill = GridBagConstraints.BOTH;
		c2.weightx = 1;
		c2.weighty = 1;
		c2.gridx = 0;
		tb = BorderFactory.createTitledBorder("Max. Radius");
		maxRField = new JTextField("       ");
		maxRField.setEditable(editable);
		if(!editable){
			maxRField.setBackground(getBackground());
			maxRField.setForeground(Color.gray.darker());
		} else
			maxRField.setBackground(Globals.INPUT_BG);
		l = new JPanel(new GridLayout());
		l.add(maxRField);
		l.setBorder(tb);
		gridbag2.setConstraints(l,c2);
		p.add(l);
		c2.gridx = 1;
		tb = BorderFactory.createTitledBorder("Grid Size");
		sizeField = new JTextField("       ");
		sizeField.setEditable(editable);
		if(!editable){
			sizeField.setBackground(getBackground());
			sizeField.setForeground(Color.gray.darker());
		} else
			sizeField.setBackground(Globals.INPUT_BG);
		l = new JPanel(new GridLayout());
		l.add(sizeField);
		l.setBorder(tb);
		gridbag2.setConstraints(l,c2);
		p.add(l);
		c2.gridx = 2;
		tb = BorderFactory.createTitledBorder("Precision");
		precField = new JTextField("--- digits");
		precField.setEditable(false);
		precField.setForeground(Color.gray.darker());
		l = new JPanel(new GridLayout());
		l.add(precField);
		l.setBorder(tb);
		gridbag2.setConstraints(l,c2);
		p.add(l);
		c2.gridx = 3;
		gridbag.setConstraints(p,c);
		tb = BorderFactory.createTitledBorder("Time");
		timeField = new JTextField("--- sec");
		timeField.setEditable(false);
		timeField.setForeground(Color.gray.darker());
		l = new JPanel(new GridLayout());
		l.add(timeField);
		l.setBorder(tb);
		gridbag2.setConstraints(l,c2);
		p.add(l);
		gridbag.setConstraints(p,c);
		add(p);
		estimateButton = new JButton("Estimate");
		estimateButton.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseReleased(MouseEvent arg0) {
				Container topLevel = BaseLightPatternEditor.this.getTopLevelAncestor();
				topLevel.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				try {
					P lp = getLightPattern();
					estimator.setParameters(lp);
					setPrecision(lp.getAccuracyUpperBound());
					setTime(lp.estimateCalcTime());
				} catch(Exception e) {
					JOptionPane.showMessageDialog(BaseLightPatternEditor.this,e.getMessage());
				}
				topLevel.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			}
		});
	}
	abstract void setLightPattern(P lp);
	abstract P getLightPattern() throws Exception;
	abstract int initGui(GridBagLayout gb,GridBagConstraints c,int row);
	void clearEstimation() {
		setPrecision(-1);
		setTime(-1);
	}
	JButton[] getEditorButtons() {
		return new JButton[]{estimateButton};
	}
	T getMaxRadius() throws Exception {
		String txt = maxRField.getText().trim();
		if(txt==null||txt.compareTo("")==0)
			throw new Exception("Maximum radius is missing.");
		int pos = txt.indexOf("%");
		if(pos!=-1)
			return math.util.parse(txt.substring(0,pos).trim());
		throw new Exception("Percent unit for the radius is missing.");
	}
	void setMaxRadius(T mr) {
		String maxR = math.util.toPlainString(mr);
		maxRField.setText(maxR+" %");
	}
	short getGridSize() throws Exception {
		String txt = sizeField.getText().trim();
		if(txt==null||txt.compareTo("")==0)
			throw new Exception("Grid size is missing.");
		int pos = txt.indexOf("x");
		if(pos!=-1)
			return Short.parseShort(txt.substring(0,pos));
		return Short.parseShort(txt);
	}
	void setGridSize(short gs) {
		sizeField.setText(gs+"x"+gs);
	}
	String getLightPatternName() {
		return nameField.getText().trim();
	}
	void setLightPatternName(String name) {
		nameField.setText(name);
	}
	void setPrecision(int p) {
		if(precField!=null) {
			if(p==-1)
				precField.setText("--- digits");
			else
				precField.setText(p+" digits");
		}
	}
	void setTime(int t) {
		if(timeField!=null) {
			if(t==-1)
				timeField.setText("--- sec");
			else if(t<60)
				timeField.setText(t+" sec");
			else if(t<3600)
				timeField.setText(((int)(t*10f/60))/10f+" min");
			else if(t<24*3600)
				timeField.setText(((int)(t*10f/3600))/10f+" h");
			else
				timeField.setText(((int)(t*10f/(24*3600)))/10f+" d");
		}
	}
	@Override
	public void setEnabled(boolean b) {
		super.setEnabled(b);
		nameField.setEnabled(b);
		sizeField.setEnabled(b);
		maxRField.setEnabled(b);
		precField.setEnabled(b);
		timeField.setEnabled(b);
	}
	public void clear() {
		setMaxRadius(math.ZERO);
		setGridSize((short)0);
		setLightPatternName("");
		setPrecision(-1);
		setTime(-1);
	}
	public void setEstimator(Estimator e) {
		this.estimator = e;
	}
	final class Estimator {

		private Controller<T> ctrl;
		Estimator(Controller<T> c) {
			this.ctrl = c;
		}
		void setParameters(ILightPattern<T> lp) {
			Model<T> model = ctrl.getModel();
			T gap = model.calcGap(lp.getGridSize());
			T size = model.calcShapeSize(lp.getGridSize());
			T[] corner = model.getGridInitBound().getCorner().clone();
			lp.setParameters(corner,gap,size);
		}
	}
}
