/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc.mgr;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JToggleButton;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import math.VecMath;
import mvc.Controller;
import mvc.IModelListener;
import mvc.LightPatternSelector;
import mvc.Model;
import pattern.BaseLightPattern;
import pattern.CalculationRecords;
import pattern.ILightPattern;
import pattern.LightChar;
import pattern.LightCharPattern;

import common.FileIO;
import common.Log;
import common.SelectTextItemListener;

@SuppressWarnings("serial")
public class LightPatternManager<T extends Number> extends JFrame {

	private JTree patternTree;
	private BaseLightPatternEditor<T,BaseLightPattern<T>> artEditor;
	private BaseLightPatternEditor<T,LightCharPattern<T>> textEditor;
	private final Controller<T> ctrl;
	private JPanel buttonPanel;
	private JScrollPane patternTreeScroller;
	private IModelListener modelListener;
	private JToggleButton help;
	public LightPatternManager(Controller<T> c) {
		super("Pattern Manager");
		this.ctrl = c;
		buildGuiComponents();
		layoutGui(artEditor);
		TreeNode root = (TreeNode)patternTree.getModel().getRoot();
		TreeNode c0 = root.getChildAt(0);
		if(c0.getChildCount()>0) {
			TreeNode c00 = c0.getChildAt(0);
			TreePath path1 = new TreePath(new TreeNode[]{root,c0,c00});
			patternTree.setSelectionPath(path1);
		} else {
			TreePath path1 = new TreePath(new TreeNode[]{root,c0});
			patternTree.setSelectionPath(path1);
		}
		TreeNode c1 = root.getChildAt(1);
		TreePath path2 = new TreePath(new TreeNode[]{root,c1});
		patternTree.expandPath(path2);
	}
	public LightPatternManager(Controller<T> c, boolean artOrMsg) {
		super("Pattern Manager");
		this.ctrl = c;
		buildGuiComponents();
		layoutGui(artEditor);
		TreeNode root = (TreeNode)patternTree.getModel().getRoot();
		TreeNode c0 = root.getChildAt(0);
		TreeNode c1 = root.getChildAt(1);
		if(artOrMsg){
			if(c0.getChildCount()>0) {
				TreeNode c00 = c0.getChildAt(0);
				TreePath path1 = new TreePath(new TreeNode[]{root,c0,c00});
				patternTree.setSelectionPath(path1);
			} else {
				TreePath path1 = new TreePath(new TreeNode[]{root,c0});
				patternTree.setSelectionPath(path1);
			}
		} else
			if(c1.getChildCount()>0) {
				TreeNode c10 = c1.getChildAt(0);
				TreePath path1 = new TreePath(new TreeNode[]{root,c1,c10});
				patternTree.setSelectionPath(path1);
			} else {
				TreePath path1 = new TreePath(new TreeNode[]{root,c1});
				patternTree.setSelectionPath(path1);
			}
	}
	private void buildGuiComponents() {
		buildTree();
		Model<T> m = ctrl.getModel();
		artEditor = new ArtLightPatternEditor<T>(m.getMath());
		artEditor.setEstimator(artEditor.new Estimator(ctrl));
		textEditor = new TextLightPatternEditor<T>(m.getLightChars(),m.getMath());
		textEditor.setEstimator(textEditor.new Estimator(ctrl));
		modelListener = new IModelListener() {

			@Override
			public void modelChanged(byte event) {
				if(event==Model.SHAPE_RESIZE) {
					artEditor.setPrecision(-1);
					artEditor.setTime(-1);
					textEditor.setPrecision(-1);
					textEditor.setTime(-1);
				}
			}
		};
		m.addModelListener(modelListener);
		buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		help = new JToggleButton("Help");
		String text = new FileIO<T>(null).loadInfoText("Manager");
		help.addItemListener(new SelectTextItemListener(help,"Help",text,600,600));
		JButton calc = new JButton("Calculate");
		calc.addActionListener(new ActionListener() {
			@Override
			@SuppressWarnings("unchecked")
			public void actionPerformed(ActionEvent ae) {
				Component ed = getContentPane().getComponent(1);
				Object o = patternTree.getLastSelectedPathComponent();
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)o;
				BaseLightPattern<T> lpOrig = 
					(BaseLightPattern<T>)node.getUserObject();
				LightPatternSelector<T> lps = 
					ctrl.getView().getLightPatternSelector();
				try {
					BaseLightPattern<T> lp = 
						((BaseLightPatternEditor<T,?>)ed).getLightPattern();
					lps.select(LightPatternManager.this,lp,lp.isStoreCalcEnabled()
					        &&lp.equals(lpOrig));
				} catch(Exception e) {
					JOptionPane.showMessageDialog(ctrl.getView(),e.getMessage());
				}
			}
		});
		JButton save = new JButton("Save");
		save.addActionListener(new SaveActionListener());
		buttonPanel.add(help);
		buttonPanel.add(calc);
		buttonPanel.add(save);
	}
	private void layoutGui(BaseLightPatternEditor<T,?> editor) {
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		getContentPane().setLayout(gridbag);
		c.insets = new Insets(5,5,5,5);
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0;
		c.weighty = 1;
		c.gridwidth = 1;
		c.gridheight = 2;
		gridbag.setConstraints(patternTreeScroller,c);
		getContentPane().add(patternTreeScroller);
		c.insets = new Insets(5,0,0,5);
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 10;
		c.weighty = 1;
		c.gridwidth = 1;
		c.gridheight = 1;
		gridbag.setConstraints(editor,c);
		getContentPane().add(editor);
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 1;
		c.gridy = 1;
		c.weightx = 1;
		c.weighty = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		JButton[] eb = editor.getEditorButtons();
		if(eb!=null)
			for(JButton b:eb)
				buttonPanel.add(b);
		gridbag.setConstraints(buttonPanel,c);
		getContentPane().add(buttonPanel);
	}
	@Override
	public void dispose() {
		ctrl.getModel().removeModelListener(modelListener);
		help.setSelected(false);
	}
	private void buildTree() {
		List<BaseLightPattern<T>> patterns = ctrl.getModel().getLightPatterns();
		final TreeNode root = new DefaultMutableTreeNode("Light Patterns");
		final TreeNode artNode = new DefaultMutableTreeNode("Symmetric");
		final TreeNode txtNode = new DefaultMutableTreeNode("Symbol Sequence");
		((DefaultMutableTreeNode)root).add((MutableTreeNode)artNode);
		((DefaultMutableTreeNode)root).add((MutableTreeNode)txtNode);
		ArrayList<ILightPattern<T>> artLPList = new ArrayList<ILightPattern<T>>();
		ArrayList<ILightPattern<T>> txtLPList = new ArrayList<ILightPattern<T>>();
		for(ILightPattern<T> p:patterns)
			if(p instanceof LightCharPattern)
				txtLPList.add(p);
			else
				artLPList.add(p);
		Comparator<ILightPattern<T>> comp = new Comparator<ILightPattern<T>>() {

			public int compare(ILightPattern<T> lp0,ILightPattern<T> lp1) {
				return lp0.getName().compareTo(lp1.getName());
			}
		};
		Collections.sort(artLPList,comp);
		Collections.sort(txtLPList,comp);
		for(ILightPattern<T> p:txtLPList)
			((DefaultMutableTreeNode)txtNode).add(new LightPatternNode(p));
		for(ILightPattern<T> p:artLPList)
			((DefaultMutableTreeNode)artNode).add(new LightPatternNode(p));
		final DefaultTreeModel model = new DefaultTreeModel(root);
		patternTree = new JTree(model);
		patternTreeScroller = new JScrollPane(patternTree);
		patternTreeScroller.setHorizontalScrollBarPolicy(
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		Dimension d = patternTree.getPreferredSize();
		d.width *= 1.5;
		patternTreeScroller.setMinimumSize(d);
		patternTreeScroller.setPreferredSize(d);
		patternTreeScroller.setMaximumSize(d);
		JPopupMenu popupMenu = new JPopupMenu();
		final JMenuItem newItem = new JMenuItem("New");
		newItem.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseReleased(MouseEvent me) {
				TreeNode node = (TreeNode)patternTree.getLastSelectedPathComponent();
				if(node!=null) {
					VecMath<T> math = ctrl.getModel().getMath();
					BaseLightPattern<T> p = null;
					int nc = nameCount("New",(DefaultMutableTreeNode)root);
					String name = "New "+(nc==0?"":nc);
					if(node==artNode) {
						p = new BaseLightPattern<T>(name,math);
					} else {
						LightChar[] lightChars = ctrl.getModel().getLightChars();
						try {
							p = new LightCharPattern<T>(name,"",lightChars,math);
						} catch(Exception e) {
							Log.getIstc().log(e);
						}
					}
					DefaultMutableTreeNode child = new LightPatternNode(p);
					sortedInsert(model,(MutableTreeNode)node,child);
					TreePath path = new TreePath(new Object[]{root,node,child});
					patternTree.setSelectionPath(path);
					int row = patternTree.getRowForPath(path);
					Point loc = patternTree.getRowBounds(row).getLocation();
					loc.x = patternTree.getBounds().getLocation().x;
					patternTreeScroller.getViewport().setViewPosition(loc);
				}
			}
		});
		final JMenuItem copyItem = new JMenuItem("Duplicate");
		copyItem.addMouseListener(new MouseAdapter() {

			@Override
			@SuppressWarnings("unchecked")
			public void mouseReleased(MouseEvent me) {
				Object lspc = patternTree.getLastSelectedPathComponent();
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)lspc;
				BaseLightPattern<T> origPat = (BaseLightPattern<T>)node.getUserObject();
				int nc = nameCount(origPat.getName(),(DefaultMutableTreeNode)root);
				String name = origPat.getName()+" "+(nc==0?"":nc);
				BaseLightPattern<T> newPat = null;
				if(node.getParent()==artNode) {
					newPat = new BaseLightPattern<T>(origPat);
					newPat.setName(name);
				} else {
					try {
						LightCharPattern<T> origLcp = (LightCharPattern<T>)origPat;
						newPat = new LightCharPattern<T>(origLcp);
						newPat.setName(name);
					} catch(Exception e) {
						Log.getIstc().log(e);
					}
				}
				DefaultMutableTreeNode child = new LightPatternNode(newPat);
				sortedInsert(model,(MutableTreeNode)node.getParent(),child);
			}
		});
		final JMenuItem deleteItem = new JMenuItem("Delete");
		deleteItem.addMouseListener(new MouseAdapter() {

			@Override
			@SuppressWarnings("unchecked")
			public void mouseReleased(MouseEvent me) {
				Object lspc = patternTree.getLastSelectedPathComponent();
				DefaultMutableTreeNode n = (DefaultMutableTreeNode)lspc;
				ILightPattern<T> pat = (ILightPattern<T>)n.getUserObject();
				String msg = "Delete Pattern "+pat.getName()+" ?";
				int choice = JOptionPane.showConfirmDialog(LightPatternManager.this,msg);
				if(choice==JOptionPane.OK_OPTION) {
					DefaultMutableTreeNode p = (DefaultMutableTreeNode)n.getParent();
					int pos = p.getIndex(n);
					p.remove(n);
					model.nodesWereRemoved(p,new int[]{pos},new Object[]{n});
					ctrl.getModel().removeLightPattern(pat);
					setEditingEnabled(false);
				}
			}
		});
		popupMenu.add(newItem);
		popupMenu.add(copyItem);
		popupMenu.add(deleteItem);
		patternTree.setComponentPopupMenu(popupMenu);
		patternTree.addTreeSelectionListener(new TreeSelectionListener() {

			@Override
			@SuppressWarnings("unchecked")
			public void valueChanged(TreeSelectionEvent tse) {
				Object node = patternTree.getLastSelectedPathComponent();
				if(root==node||null==node) {
					newItem.setEnabled(false);
					copyItem.setEnabled(false);
					deleteItem.setEnabled(false);
					setEditingEnabled(false);
				} else if(artNode==node||txtNode==node) {
					newItem.setEnabled(true);
					copyItem.setEnabled(false);
					deleteItem.setEnabled(false);
					setEditingEnabled(false);
				} else {
					newItem.setEnabled(false);
					copyItem.setEnabled(true);
					deleteItem.setEnabled(true);
				}
				if(node!=null) {
					Object o = ((DefaultMutableTreeNode)node).getUserObject();
					if(o instanceof BaseLightPattern)
						edit((BaseLightPattern<T>)o);
				}
			}
		});
	}
	@SuppressWarnings("unchecked")
	private static void sortedInsert(DefaultTreeModel model,
			MutableTreeNode par, DefaultMutableTreeNode child) {
		String childName = ((ILightPattern)child.getUserObject()).getName();
		int pos = par.getChildCount();
		for(int i = 0;i<par.getChildCount();i++) {
			Object o = ((DefaultMutableTreeNode)par.getChildAt(i)).getUserObject();
			if(((ILightPattern)o).getName().compareTo(childName)>=0) {
				pos = i;
				break;
			}
		}
		par.insert(child,pos);
		model.nodesWereInserted(par,new int[]{par.getIndex(child)});
	}
	@SuppressWarnings("unchecked")
	private int nameCount(String name,DefaultMutableTreeNode root) {
		int cnt = -1;
		Enumeration<DefaultMutableTreeNode> en = root.depthFirstEnumeration();
		while(en.hasMoreElements()) {
			Object o = en.nextElement().getUserObject();
			if(o instanceof ILightPattern) {
				ILightPattern<T> p = (ILightPattern<T>)o;
				if(p.getName().startsWith(name)) {
					String strCtr = p.getName().trim().substring(name.length());
					try {
						int ctr = Integer.parseInt(strCtr.trim());
						if(cnt<ctr)
							cnt = ctr;
					} catch(Exception e) {
						cnt = cnt==-1?0:cnt;
					}
				}
			}
		}
		return cnt+1;
	}
	private void setEditingEnabled(boolean b) {
		for(Component c:buttonPanel.getComponents())
			c.setEnabled(b);
		Component editor = getContentPane().getComponent(1);
		if(!b)
			((BaseLightPatternEditor)editor).clear();
		editor.setEnabled(b);
	}
	@SuppressWarnings("unchecked")
	private void edit(BaseLightPattern<T> lp) {
		BaseLightPatternEditor editor = artEditor;
		if(lp instanceof LightCharPattern)
			editor = textEditor;
		Component c = getContentPane().getComponent(1);
		BaseLightPatternEditor prevEditor = (BaseLightPatternEditor)c;
		if(prevEditor!=editor) {
			Dimension size = getSize();
			JButton[] eb = prevEditor.getEditorButtons();
			if(eb!=null) {
				for(JButton b:eb)
					buttonPanel.remove(b);
				buttonPanel.invalidate();
			}
			getContentPane().removeAll();
			layoutGui(editor);
			getContentPane().invalidate();
			setSize(new Dimension(size.width+1,size.height+1));
		}
		setEditingEnabled(true);
		editor.setLightPattern(lp);
	}
	private final class SaveActionListener implements ActionListener {
		@Override
		@SuppressWarnings("unchecked")
		public void actionPerformed(ActionEvent ae) {
			Component ed = getContentPane().getComponent(1);
			try {
				BaseLightPatternEditor<T,?> blped = 
					(BaseLightPatternEditor<T,?>)ed;
				ILightPattern<T> lp = blped.getLightPattern();
				Object o = patternTree.getLastSelectedPathComponent();
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)o;
				ILightPattern<T> lpOrig = (ILightPattern<T>)node.getUserObject();
				CalculationRecords<T> records = null;
				if(lpOrig.equals(lp))
					records = ctrl.getModel().getRecords(lpOrig.getName());
				if(lpOrig.getName().compareTo(lp.getName())!=0) {
					TreeNode root = (TreeNode)patternTree.getModel().getRoot();
					for(int j = 0;j<root.getChildCount();j++) {
						MutableTreeNode c = (MutableTreeNode)root.getChildAt(j);
						for(int i = 0;i<c.getChildCount();i++) {
							DefaultMutableTreeNode cc = 
								(DefaultMutableTreeNode)c.getChildAt(i);
							ILightPattern<T> lipat = 
								(ILightPattern<T>)cc.getUserObject();
							if(lipat.getName().compareTo(lp.getName())==0)
								throw new Exception("Name is already in use.");
						}
					}
					ctrl.getModel().removeLightPattern(lpOrig);
				}
				ctrl.getModel().addLightPattern(lp);
				if(records!=null) {
					records.setPatternName(lp.getName());
					ctrl.getModel().addRecords(records);
				} else
					ctrl.getModel().deleteRecords(lp.getName());
				node.setUserObject(lp);
				((DefaultTreeModel)patternTree.getModel()).nodeChanged(node);
			} catch(Exception e) {
				JOptionPane.showMessageDialog(ctrl.getView(),e.getMessage());
			}
		}
	}
	private class LightPatternNode extends DefaultMutableTreeNode {
		public LightPatternNode(ILightPattern blp){
			super(blp);
		}
		@Override
		public String toString(){
			return ((BaseLightPattern)getUserObject()).getName();
		}
	}
}
