/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc.mgr;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.Border;

import math.VecMath;
import pattern.LightChar;
import pattern.LightCharPattern;

import common.Globals;

public class TextLightPatternEditor<T extends Number>
        extends
            BaseLightPatternEditor<T,LightCharPattern<T>> {

	private JEditorPane textEditor;
	private TextPreviewCanvas textPreview;
	private final LightChar[] lightChars;
	private JCheckBox storeCalc;
	private JScrollPane previewScroller;
	private final T maxR;
	public TextLightPatternEditor(LightChar[] lcs,VecMath<T> mt) {
		super(false,2,mt);
		this.lightChars = lcs;
		this.maxR = mt.util.cast(LightCharPattern.MAX_RADIUS);
		setMaxRadius(maxR);
	}
	@Override
	void setLightPattern(LightCharPattern<T> lcp) {
		setLightPatternName(lcp.getName());
		textEditor.setText(lcp.getText());
		storeCalc.setSelected(lcp.isStoreCalcEnabled());
		textChanged();
	}
	@Override
	LightCharPattern<T> getLightPattern() throws Exception {
		String txt = textEditor.getText();
		boolean store = storeCalc.isSelected();
		String name = getLightPatternName();
		return new LightCharPattern<T>(name,txt,store,lightChars,math);
	}
	private void textChanged() {
		String rText = LightCharPattern.reduce(textEditor.getText());
		short gSize = LightCharPattern.calcGridSize(rText);
		setGridSize(gSize);
		textPreview.setText(rText,gSize);
		previewScroller.repaint();
		clearEstimation();
	}
	@Override
	int initGui(GridBagLayout gbl,GridBagConstraints gbc,int row) {
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = row;
		gbc.weightx = 1;
		gbc.weighty = 15;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		Border tb = BorderFactory.createTitledBorder("Text");
		textEditor = new JEditorPane();
		textEditor.setBackground(Globals.INPUT_BG);
		textEditor.setFont(new Font(Font.MONOSPACED,Font.PLAIN,9));
		textEditor.addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent arg0) {
				textChanged();
			}
		});
		textEditor.setPreferredSize(new Dimension(100,100));
		JScrollPane sp = new JScrollPane(textEditor);
		int policy = ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER;
		sp.setHorizontalScrollBarPolicy(policy);
		policy = ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
		sp.setVerticalScrollBarPolicy(policy);
		sp.setBorder(tb);
		sp.setPreferredSize(new Dimension(100,100));
		gbl.setConstraints(sp,gbc);
		add(sp);
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 1;
		gbc.gridy = row;
		gbc.weightx = 1;
		gbc.weighty = 15;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		tb = BorderFactory.createTitledBorder("Layout");
		textPreview = new TextPreviewCanvas();
		textPreview.setFont(new Font(Font.MONOSPACED,Font.PLAIN,8));
		previewScroller = new JScrollPane(textPreview);
		policy = ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER;
		previewScroller.setHorizontalScrollBarPolicy(policy);
		policy = ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
		previewScroller.setVerticalScrollBarPolicy(policy);
		previewScroller.setBorder(tb);
		previewScroller.setPreferredSize(new Dimension(100,100));
		gbl.setConstraints(previewScroller,gbc);
		add(previewScroller);
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = ++row;
		gbc.weightx = 1;
		gbc.weighty = 0;
		gbc.gridwidth = columns;
		gbc.gridheight = 1;
		tb = BorderFactory.createTitledBorder("Store Calculations");
		GridBagLayout gbl2 = new GridBagLayout();
		GridBagConstraints gbc2 = new GridBagConstraints();
		JPanel p = new JPanel(gbl2);
		gbc2.fill = GridBagConstraints.BOTH;
		gbc2.gridx = 0;
		gbc2.weightx = 0.1;
		gbc2.weighty = 1;
		String txt = "Avoids recalculation of the pattern but reveals"+
                     " the radius in the storage file.";
		JTextPane tp = new JTextPane();
		tp.setBackground(getBackground());
		tp.setText(txt);
		tp.setPreferredSize(new Dimension(100,35));
		storeCalc = new JCheckBox("");
		gbl2.setConstraints(storeCalc,gbc2);
		p.add(storeCalc);
		gbc2.gridx = 1;
		gbc2.weightx = 1;
		gbl2.setConstraints(tp,gbc2);
		p.add(tp);
		p.setBorder(tb);
		gbl.setConstraints(p,gbc);
		add(p);
		return row+1;
	}
	@Override
	public void setEnabled(boolean b) {
		super.setEnabled(b);
		textEditor.setEnabled(b);
		textPreview.setEnabled(b);
		storeCalc.setEnabled(b);
	}
	@Override
	public void clear() {
		super.clear();
		setMaxRadius(maxR);
		textEditor.setText("");
		textPreview.setText("",(short)-1);
		storeCalc.setSelected(false);
	}
	private class TextPreviewCanvas extends JPanel {

		private String text;
		private short size;
		void setText(String txt,short gridSize) {
			this.text = txt;
			this.size = gridSize;
			int txtHeight = getFontMetrics(textPreview.getFont()).getHeight()-2;
			int txtWidth = getFontMetrics(textPreview.getFont()).charWidth('M');
			setPreferredSize(new Dimension(txtWidth*size,txtHeight*size));
			revalidate();
		}
		@Override
		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			if(text!=null) {
				Graphics2D g2d = (Graphics2D)g;
				String[] lines = LightCharPattern.preview(text,size);
				int txtHeight = getFontMetrics(textPreview.getFont()).getHeight()-2;
				int txtWidth = getFontMetrics(textPreview.getFont()).charWidth('M');
				float cols = (float)(getSize().getWidth()/txtWidth);
				float rows = (float)(getSize().getHeight()/txtHeight);
				int x = (int)((cols-(size-1))/2*txtWidth);
				int y = (int)((rows-(size-1))/2*txtHeight);
				g2d.setFont(getFont());
				for(int i = 0;i<lines.length;i++) {
					if(lines[i]!=null)
						g2d.drawString(lines[i],x,y+(i+1)*txtHeight);
				}
			}
		}
	}
}
