/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc.scan;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JRadioButtonMenuItem;

import common.FileIO;
import common.SelectTextItemListener;


@SuppressWarnings("serial")
public class ScanMenuBar<T extends Number> extends JMenuBar {

	private JMenu obsMenu;
	private JMenu runMenu;
	private final ScanVisualizer<T> vis;
	private JRadioButtonMenuItem runItem;
	private ScanListener scanListener;
	private boolean allowEvent = true;
	private JCheckBoxMenuItem infoItem;
	public ScanMenuBar(ScanVisualizer<T> vis) {
		this.vis = vis;
		init();
	}
	private void init() {
		JMenu infoMenu = new JMenu("?");
		infoItem = new JCheckBoxMenuItem("help...");
		String text = new FileIO<T>(null).loadInfoText("Scanner");
		String title = "Scanner Info";
		infoItem.addItemListener(new SelectTextItemListener(infoItem,title,text,550,450));
		obsMenu = new JMenu("Observables");
		runItem = new JRadioButtonMenuItem("Run");
		runMenu = new JMenu("Run");
		runItem.addItemListener(new ItemListener() {

			@Override
			public void itemStateChanged(ItemEvent e) {
				if(allowEvent) {
					if(e.getStateChange()==ItemEvent.SELECTED)
						vis.startScan();
					else
						vis.stopScan("Stopped");
				}
			}
		});
		scanListener = new ScanListener() {

			@Override
			public void start() {}
			@Override
			public void stop(String msg) {
				if(runItem.isSelected()) {
					allowEvent = false;
					runItem.setSelected(false);
					allowEvent = true;
				}
			}
			@Override
			public void update(int i,int j) {}
		};
		vis.addScanListener(scanListener);
		JRadioButtonMenuItem item11 = new JRadioButtonMenuItem("Reflections");
		item11.addItemListener(new ObsListener(Observables.OBS_REFL_CNT));
		item11.setSelected(true);
		JRadioButtonMenuItem item12 = new JRadioButtonMenuItem("Length");
		item12.addItemListener(new ObsListener(Observables.OBS_LENGTH));
		JRadioButtonMenuItem item13 = new JRadioButtonMenuItem("Reflections/Length");
		item13.addItemListener(new ObsListener(Observables.OBS_OBS1));
		add(runMenu);
		runMenu.add(runItem);
		add(obsMenu);
		infoMenu.add(infoItem);
		add(infoMenu);
		obsMenu.add(item11);
		obsMenu.add(item12);
		obsMenu.add(item13);
		ButtonGroup bg = new ButtonGroup();
		bg.add(item11);
		bg.add(item12);
		bg.add(item13);
	}
	private final class ObsListener implements ItemListener {

		private final byte obs;
		public ObsListener(byte o) {
			this.obs = o;
		}
		public void itemStateChanged(ItemEvent ie) {
			if(ie.getStateChange()==ItemEvent.SELECTED)
				vis.setObservable(obs);
		}
	}
	public void postInit() {
		runItem.setSelected(true);
	}
	public void onDelete() {
		vis.removeScanListener(scanListener);
		infoItem.setSelected(false);
	}
}
