/**
	Copyright (c) 2009-2012 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package test;
import java.util.ArrayList;

import junit.framework.TestCase;
import lattice.Lattice;
import lattice.Ray;
import math.VecMath;
import math.VecMathApfloat;
import math.VecMathFloat;

import org.apfloat.Apfloat;
import org.junit.Test;

public class LatticeTest {

	private static final Float DELTA = 0.000001f;
	private static final VecMathFloat math = new VecMathFloat();
	@Test
	public void test2DIntersection() {
		Float[] corner = new Float[]{-1.0f,-1.0f};
		Lattice<Float> lattice = new Lattice<Float>(corner,(short)11,0.2f,math);
		Float[] start = new Float[]{2.0f,0.05f};
		Float[] dir = new Float[]{-1.0f,0.0f};
		Ray<Float> ray = new Ray<Float>(start,dir,null);
		ArrayList<Float[]> cellCenters = new ArrayList<Float[]>();
		lattice.intersect(ray,cellCenters);
		TestCase.assertEquals("Cells",10,cellCenters.size());
		for(int i = 0;i<cellCenters.size();i++) {
			// System.out.println ("Cell"+i+" x: "+cellCenters.get(i)[0]);
			// System.out.println ("Cell"+i+" y: "+cellCenters.get(i)[1]);
			// System.out.println ("Cell"+i+" z: "+cellCenters.get(i)[2]);
			TestCase.assertEquals("Cell"+i+" x",0.9-i*0.2,cellCenters.get(i)[0],DELTA);
			TestCase.assertEquals("Cell"+i+" y",0.1,cellCenters.get(i)[1],DELTA);
		}
		lattice.rotate(math.ZERO2D,null,(float)Math.PI/2);
		cellCenters.clear();
		lattice.intersect(ray,cellCenters);
		TestCase.assertEquals("Cells",10,cellCenters.size());
		for(int i = 0;i<cellCenters.size();i++) {
			// System.out.println ("Cell"+i+" x: "+cellCenters.get(i)[0]);
			// System.out.println ("Cell"+i+" y: "+cellCenters.get(i)[1]);
			// System.out.println ("Cell"+i+" z: "+cellCenters.get(i)[2]);
			TestCase.assertEquals("Cell"+i+" x",0.9-i*0.2,cellCenters.get(i)[0],DELTA);
			TestCase.assertEquals("Cell"+i+" y",0.1,cellCenters.get(i)[1],DELTA);
		}
		// Rotate lattice with pi/4
		corner = new Float[]{-1.0f,-1.0f};
		lattice = new Lattice<Float>(corner,(short)11,0.2f,math);
		Float[] sincos = math.sincos((float)Math.PI/4);
		lattice.rotate(math.ZERO2D,math.ZERO2D,sincos[0],sincos[1]);
		start = new Float[]{-2.0f,0.01f};
		dir = new Float[]{1.0f,0.0f};
		ray = new Ray<Float>(start,dir,null);
		cellCenters.clear();
		lattice.intersect(ray,cellCenters);
		Float d = math.mul(math.sqrt(math.TWO),lattice.getGap());
		// System.out.println("d: "+d.doubleValue());
		Float[] x_d = new Float[10];
		Float[] y_d = new Float[10];
		for(int i = 0;i<y_d.length;i++) {
			x_d[i] = math.add(math.mul(d,i-5),math.div(d,math.TWO));
			y_d[i] = math.ZERO;
		}
		Float[] x_u = new Float[9];
		Float[] y_u = new Float[9];
		for(int i = 0;i<y_u.length;i++) {
			x_u[i] = math.mul(d,i-4);
			y_u[i] = math.div(d,math.TWO);
		}
		TestCase.assertEquals("Cells",19,cellCenters.size());
		boolean updown = false;
		for(int i = 0;i<cellCenters.size();i++) {
			Float x = updown?x_u[i/2]:x_d[i/2];
			Float y = updown?y_u[i/2]:y_d[i/2];
			TestCase.assertEquals("Cell"+i+" x",x,cellCenters.get(i)[0],10*math.EPSILON);
			TestCase.assertEquals("Cell"+i+" y",y,cellCenters.get(i)[1],10*math.EPSILON);
			updown = !updown;
		}
	}
	@Test
	public void test3DIntersection() {
		Float[] corner = new Float[]{-1.0f,-1.0f,-1.0f};
		Lattice<Float> lattice = new Lattice<Float>(corner,(short)11,0.2f,math);
		Float[] start = new Float[]{0.0f,-2.0f,0.0f};
		Float[] dir = new Float[]{0.0f,1.0f,0.0f};
		Ray<Float> ray = new Ray<Float>(start,dir,null);
		ArrayList<Float[]> cellCenters = new ArrayList<Float[]>();
		lattice.intersect(ray,cellCenters);
		TestCase.assertEquals("Cells",10,cellCenters.size());
		for(int i = 0;i<cellCenters.size();i++) {
			// System.out.println ("Cell"+i+" x: "+cellCenters.get(i)[0]);
			// System.out.println ("Cell"+i+" y: "+cellCenters.get(i)[1]);
			// System.out.println ("Cell"+i+" z: "+cellCenters.get(i)[2]);
			TestCase.assertEquals("Cell"+i+" x",0.1,Math.round(10*cellCenters.get(i)[0])/10d);
			TestCase.assertEquals("Cell"+i+" y",Math.round(10*(-0.9+i*0.2))/10d,Math
			        .round(10*cellCenters.get(i)[1])/10d);
			TestCase.assertEquals("Cell"+i+" z",0.1,Math.round(10*cellCenters.get(i)[2])/10d);
		}
		start = new Float[]{2.0f,0.5f,0.5f};
		dir = new Float[]{-1.0f,0.0f,0.0f};
		ray = new Ray<Float>(start,dir,null);
		cellCenters.clear();
		lattice.intersect(ray,cellCenters);
		TestCase.assertEquals("Cells",10,cellCenters.size());
		for(int i = 0;i<cellCenters.size();i++) {
			// System.out.println ("Cell"+i+" x: "+cellCenters.get(i)[0]);
			// System.out.println ("Cell"+i+" y: "+cellCenters.get(i)[1]);
			// System.out.println ("Cell"+i+" z: "+cellCenters.get(i)[2]);
			TestCase.assertEquals("Cell"+i+" x",Math.round(10*(0.9-i*0.2))/10d,Math
			        .round(10*cellCenters.get(i)[0])/10d);
			TestCase.assertEquals("Cell"+i+" y",0.5,Math.round(10*cellCenters.get(i)[1])/10d);
			TestCase.assertEquals("Cell"+i+" z",0.5,Math.round(10*cellCenters.get(i)[2])/10d);
		}
		start = new Float[]{0.5f,0.5f,2.0f};
		dir = new Float[]{0.0f,0.0f,-1.0f};
		ray = new Ray<Float>(start,dir,null);
		cellCenters.clear();
		lattice.intersect(ray,cellCenters);
		TestCase.assertEquals("Cells",10,cellCenters.size());
		for(int i = 0;i<cellCenters.size();i++) {
			TestCase.assertEquals("Cell"+i+" x",0.5,Math.round(10*cellCenters.get(i)[0])/10d);
			TestCase.assertEquals("Cell"+i+" y",0.5,Math.round(10*cellCenters.get(i)[1])/10d);
			TestCase.assertEquals("Cell"+i+" z",Math.round(10*(0.9-i*0.2))/10d,Math
			        .round(10*cellCenters.get(i)[2])/10d);
		}
		for(int sign1 = -1;sign1<=1;sign1 += 2)
			for(int sign2 = -1;sign2<=1;sign2 += 2) {
				start = new Float[]{0.0f,sign2*2.0f,sign1*1.99f};
				dir = new Float[]{0.0f,sign2*(-0.1f),sign1*(-0.1f)};
				math.scale(math.inv(math.norm(dir)),dir);
				ray = new Ray<Float>(start,dir,null);
				cellCenters.clear();
				lattice.intersect(ray,cellCenters);
				TestCase.assertEquals("Cells",19,cellCenters.size());
				for(int i = 0;i<cellCenters.size();i++) {
					int idx1 = i/2;
					int idx2 = (i+1)/2;
					TestCase.assertEquals("Cell "+i+" x",0.1,
					        Math.round(10*cellCenters.get(i)[0])/10d);
					TestCase.assertEquals("Cell "+i+" y",Math.round(sign2*10*(0.9-idx1*0.2))/10d,
					        Math.round(10*cellCenters.get(i)[1])/10d);
					TestCase.assertEquals("Cell "+i+" z",Math.round(sign1*10*(0.9-idx2*0.2))/10d,
					        Math.round(10*cellCenters.get(i)[2])/10d);
				}
			}
		for(int sign1 = -1;sign1<=1;sign1 += 2)
			for(int sign2 = -1;sign2<=1;sign2 += 2)
				for(int sign3 = -1;sign3<=1;sign3 += 2) {
					start = new Float[]{sign3*2.0f,sign2*2.0f,sign1*2.0f};
					dir = new Float[]{sign3*(-0.1f),sign2*(-0.1f),sign1*(-0.1f)};
					math.scale(math.inv(math.norm(dir)),dir);
					ray = new Ray<Float>(start,dir,null);
					cellCenters.clear();
					lattice.intersect(ray,cellCenters);
					TestCase.assertEquals("Cells",10,cellCenters.size());
					for(int i = 0;i<cellCenters.size();i++) {
						TestCase.assertEquals("Cell"+i+" x",Math.round(sign3*10*(0.9-i*0.2))/10d,
						        Math.round(10*cellCenters.get(i)[0])/10d);
						TestCase.assertEquals("Cell"+i+" y",Math.round(sign2*10*(0.9-i*0.2))/10d,
						        Math.round(10*cellCenters.get(i)[1])/10d);
						TestCase.assertEquals("Cell"+i+" z",Math.round(sign1*10*(0.9-i*0.2))/10d,
						        Math.round(10*cellCenters.get(i)[2])/10d);
					}
				}
		// corner = lattice.getCorner();
		// for (int i = 0; i < start.length; i++)
		// System.out.println("corner "+i+": "+corner[i]);
		for(int sign1 = -1;sign1<=1;sign1 += 2)
			for(int sign2 = -1;sign2<=1;sign2 += 2) {
				for(int k = 0;k<7;k++) {
					Float y = sign1*(0.05f+k*0.2f);
					start = new Float[]{0.0f,y,0.0f};
					// for (int i = 0; i < start.length; i++)
					// System.out.println("start "+i+": "+start[i]);
					dir = new Float[]{0.0f,sign1*2.0f,sign2*2.0f};
					math.scale(math.inv(math.norm(dir)),dir);
					// for (int i = 0; i < dir.length; i++)
					// System.out.println("dir "+i+": "+dir[i]);
					ray = new Ray<Float>(start,dir,null);
					cellCenters.clear();
					lattice.intersect(ray,cellCenters);
					corner = lattice.getCorner();
					// for (int i = 0; i < start.length; i++)
					// System.out.println("corner "+i+": "+corner[i]);
					if(cellCenters.size()-Math.max(9-2*k,0)<0)
						System.out.println("ERROR");
					TestCase.assertEquals("Cells",true,cellCenters.size()-Math.max(9-2*k,0)>=0);
					for(int i = 0;i<Math.max(9-2*k,0);i++) {
						int idx1 = i/2;
						int idx2 = (i+1)/2;
						TestCase.assertEquals("Remaining cells",true,cellCenters.size()>0);
						int oldSize = cellCenters.size();
						for(int j = 0;j<cellCenters.size();j++) {
							if(0.1==Math.round(10*cellCenters.get(j)[0])/10d
							        &&Math.round(sign1*10*(0.1+k*0.2+idx2*0.2))/10d==Math
							                .round(10*cellCenters.get(j)[1])/10d
							        &&Math.round(sign2*10*(0.1+idx1*0.2))/10d==Math
							                .round(10*cellCenters.get(j)[2])/10d) {
								cellCenters.remove(j);
								break;
							}
						}
						if(oldSize-cellCenters.size()!=1)
							System.out.println("ERROR");
						TestCase.assertEquals("Cell found",true,oldSize-cellCenters.size()==1);
					}
				}
			}
		for(int sign1 = -1;sign1<=1;sign1 += 2)
			for(int sign2 = -1;sign2<=1;sign2 += 2) {
				for(int k = 0;k<7;k++) {
					Float x = sign1*(0.05f+k*0.2f);
					start = new Float[]{x,0.0f,0.0f};
					dir = new Float[]{sign1*2.0f,0.0f,sign2*2.0f};
					math.scale(math.inv(math.norm(dir)),dir);
					ray = new Ray<Float>(start,dir,null);
					cellCenters.clear();
					lattice.intersect(ray,cellCenters);
					corner = lattice.getCorner();
					if(cellCenters.size()-Math.max(9-2*k,0)<0)
						System.out.println("ERROR");
					TestCase.assertEquals("Cells",true,cellCenters.size()-Math.max(9-2*k,0)>=0);
					for(int i = 0;i<Math.max(9-2*k,0);i++) {
						int idx1 = i/2;
						int idx2 = (i+1)/2;
						TestCase.assertEquals("Remaining cells",true,cellCenters.size()>0);
						int oldSize = cellCenters.size();
						for(int j = 0;j<cellCenters.size();j++) {
							if(Math.round(sign1*10*(0.1+k*0.2+idx2*0.2))/10d==Math
							        .round(10*cellCenters.get(j)[0])/10d
							        &&0.1==Math.round(10*cellCenters.get(j)[1])/10d
							        &&Math.round(sign2*10*(0.1+idx1*0.2))/10d==Math
							                .round(10*cellCenters.get(j)[2])/10d) {
								cellCenters.remove(j);
								break;
							}
						}
						if(oldSize-cellCenters.size()!=1)
							System.out.println("ERROR");
						TestCase.assertEquals("Cell found",true,oldSize-cellCenters.size()==1);
					}
				}
			}
		for(int sign1 = -1;sign1<=1;sign1 += 2)
			for(int sign2 = -1;sign2<=1;sign2 += 2) {
				for(int k = 0;k<7;k++) {
					Float z = sign1*(0.05f+k*0.2f);
					start = new Float[]{0.0f,0.0f,z};
					dir = new Float[]{0.0f,sign2*2.0f,sign1*2.0f};
					math.scale(math.inv(math.norm(dir)),dir);
					ray = new Ray<Float>(start,dir,null);
					cellCenters.clear();
					lattice.intersect(ray,cellCenters);
					if(cellCenters.size()-Math.max(9-2*k,0)<0)
						System.out.println("ERROR");
					TestCase.assertEquals("Cells",true,cellCenters.size()-Math.max(9-2*k,0)>=0);
					for(int i = 0;i<Math.max(9-2*k,0);i++) {
						int idx1 = i/2;
						int idx2 = (i+1)/2;
						TestCase.assertEquals("Remaining cells",true,cellCenters.size()>0);
						int oldSize = cellCenters.size();
						for(int j = 0;j<cellCenters.size();j++) {
							if(0.1==Math.round(10*cellCenters.get(j)[0])/10d
							        &&Math.round(sign2*10*(0.1+idx1*0.2))/10d==Math
							                .round(10*cellCenters.get(j)[1])/10d
							        &&Math.round(sign1*10*(0.1+k*0.2+idx2*0.2))/10d==Math
							                .round(10*cellCenters.get(j)[2])/10d) {
								cellCenters.remove(j);
								break;
							}
						}
						if(oldSize-cellCenters.size()!=1)
							System.out.println("ERROR");
						TestCase.assertEquals("Cell found",true,oldSize-cellCenters.size()==1);
					}
				}
			}
	}
	@Test
	public void test2DPreciseIntersection() {
		VecMathApfloat math = new VecMathApfloat((short)100);
		Apfloat min_one = math.util.copy(math.util.cast(-1),math.PRECISION);
		Apfloat[] corner = new Apfloat[]{min_one,min_one};
		Apfloat gap = math.div(math.TWO,math.TEN);
		gap = math.util.copy(gap,math.PRECISION);
		Lattice<Apfloat> lattice = new Lattice<Apfloat>(corner,(short)11,gap,math);
		Apfloat two = math.util.copy(math.util.cast(2),math.PRECISION);
		Apfloat[] start = new Apfloat[]{math.ZERO,math.neg(two)};
		Apfloat[] dir = new Apfloat[]{math.ZERO,two};
		math.scale(math.inv(math.norm(dir)),dir);
		Ray<Apfloat> ray = new Ray<Apfloat>(start,dir,null);
		ArrayList<Apfloat[]> cellCenters = new ArrayList<Apfloat[]>();
		lattice.intersect(ray,cellCenters);
		TestCase.assertEquals("Cells",10,cellCenters.size());
		for(int i = 0;i<cellCenters.size();i++) {
			assertEquals("Cell"+i+" x",math.ONE_OVER_TEN,cellCenters.get(i)[0],math.EPSILON,math);
			Apfloat y = math.add(math.mul(math.ONE_OVER_TEN,-9),math.mul(math.ONE_OVER_TEN,2*i));
			assertEquals("Cell"+i+" y",y,cellCenters.get(i)[1],math.EPSILON,math);
		}
		// Rotate lattice with pi/4
		int[] precisions = new int[]{10,100,math.PRECISION};
		for(int j = 0;j<precisions.length;j++) {
			corner = new Apfloat[]{min_one,min_one};
			gap = math.div(math.TWO,math.TEN);
			gap = math.util.copy(gap,math.PRECISION);
			lattice = new Lattice<Apfloat>(corner,(short)11,gap,math);
			Apfloat[] sincos = math.discrSinCos(0.25f);
			lattice.rotate(math.ZERO2D,math.ZERO2D,sincos[0],sincos[1]);
			two = math.util.copy(math.util.cast(2),precisions[j]);
			start = new Apfloat[]{math.neg(two),math.util.copy(math.ONE_OVER_HUNDRED,precisions[j])};
			dir = new Apfloat[]{two,math.ZERO};
			math.scale(math.inv(math.norm(dir)),dir);
			ray = new Ray<Apfloat>(start,dir,null);
			cellCenters.clear();
			lattice.intersect(ray,cellCenters);
			// for (int i = 0; i < cellCenters.size(); i++) {
			// Apfloat[] ctr = cellCenters.get(i);
			// for (int k = 0; k < ctr.length; k++) {
			// System.out.println("CELL"+i+": "+k+": "+ctr[k]);
			// }
			// }
			Apfloat d = math.mul(math.sqrt(math.util.copy(math.TWO,precisions[j])),gap);
			// System.out.println("d: "+d.doubleValue());
			Apfloat[] x_d = new Apfloat[10];
			Apfloat[] y_d = new Apfloat[10];
			for(int i = 0;i<y_d.length;i++) {
				x_d[i] = math.add(math.mul(d,i-5),math.div(d,math.TWO));
				y_d[i] = math.ZERO;
			}
			Apfloat[] x_u = new Apfloat[9];
			Apfloat[] y_u = new Apfloat[9];
			for(int i = 0;i<y_u.length;i++) {
				x_u[i] = math.mul(d,i-4);
				y_u[i] = math.div(d,math.TWO);
			}
			TestCase.assertEquals("Cells",19,cellCenters.size());
			boolean updown = false;
			for(int i = 0;i<cellCenters.size();i++) {
				Apfloat x = updown?x_u[i/2]:x_d[i/2];
				Apfloat y = updown?y_u[i/2]:y_d[i/2];
				assertEquals("Cell"+i+" x",x,cellCenters.get(i)[0],math.pow(math.ONE_OVER_TEN,
				        precisions[j]-2),math);
				assertEquals("Cell"+i+" y",y,cellCenters.get(i)[1],math.pow(math.ONE_OVER_TEN,
				        precisions[j]-2),math);
				updown = !updown;
			}
		}
	}
	@Test
	public void test3DPreciseIntersection() {
		VecMathApfloat math = new VecMathApfloat((short)100);
		Apfloat min_one = math.util.copy(math.util.cast(-1),math.PRECISION);
		Apfloat[] corner = new Apfloat[]{min_one,min_one,min_one};
		Apfloat gap = math.div(math.TWO,math.TEN);
		gap = math.util.copy(gap,math.PRECISION);
		Lattice<Apfloat> lattice = new Lattice<Apfloat>(corner,(short)11,gap,math);
		Apfloat two = math.util.copy(math.util.cast(2),math.PRECISION);
		Apfloat[] start = new Apfloat[]{math.ZERO,math.neg(two),math.ZERO};
		Apfloat[] dir = new Apfloat[]{math.ZERO,two,math.ZERO};
		math.scale(math.inv(math.norm(dir)),dir);
		Ray<Apfloat> ray = new Ray<Apfloat>(start,dir,null);
		ArrayList<Apfloat[]> cellCenters = new ArrayList<Apfloat[]>();
		lattice.intersect(ray,cellCenters);
		TestCase.assertEquals("Cells",10,cellCenters.size());
		for(int i = 0;i<cellCenters.size();i++) {
			// System.out.println ("Cell"+i+" x: "+cellCenters.get(i)[0]);
			// System.out.println ("Cell"+i+" y: "+cellCenters.get(i)[1]);
			// System.out.println ("Cell"+i+" z: "+cellCenters.get(i)[2]);
			assertEquals("Cell"+i+" x",math.ONE_OVER_TEN,cellCenters.get(i)[0],math.EPSILON,math);
			Apfloat y = math.add(math.mul(math.ONE_OVER_TEN,-9),math.mul(math.ONE_OVER_TEN,2*i));
			assertEquals("Cell"+i+" y",y,cellCenters.get(i)[1],math.EPSILON,math);
			assertEquals("Cell"+i+" z",math.ONE_OVER_TEN,cellCenters.get(i)[2],math.EPSILON,math);
		}
		// lattice.rotateStable(math.ZERO2D,math.ZERO2D,math.sinPiOv2Pow[2],math.cosPiOv2Pow[2]);
	}
	@Test
	public void testgetOptimalDirPerm() {
		Float[] corner = new Float[]{-1.0f,-1.0f};
		byte[] dirs = new byte[2];
		Lattice<Float> lattice = new Lattice<Float>(corner,(short)5,0.5f,math);
		Float[] start = new Float[]{0.6f,0.75f};
		Float[] dir = new Float[]{-1.0f,0.0f};
		Ray<Float> ray = new Ray<Float>(start,dir,null);
		lattice.getOptDirPerm(ray,dirs);
		TestCase.assertEquals("optimal dir",0,dirs[0]);
		TestCase.assertEquals("optimal dir",1,dirs[1]);
		ray.setStart(new Float[]{0.6f,0.75f});
		ray.setDir(new Float[]{-0.1f,-0.2f});
		math.normalize(ray.getDir());
		lattice.getOptDirPerm(ray,dirs);
		TestCase.assertEquals("optimal dir",0,dirs[0]);
		TestCase.assertEquals("optimal dir",1,dirs[1]);
		ray.setStart(new Float[]{0.6f,0.75f});
		ray.setDir(new Float[]{-0.1f,-0.3f});
		math.normalize(ray.getDir());
		lattice.getOptDirPerm(ray,dirs);
		TestCase.assertEquals("optimal dir",1,dirs[0]);
		TestCase.assertEquals("optimal dir",0,dirs[1]);
		ray.setStart(new Float[]{0.9f,0.9f});
		ray.setDir(new Float[]{0.0f,-1.0f});
		math.normalize(ray.getDir());
		lattice.getOptDirPerm(ray,dirs);
		TestCase.assertEquals("optimal dir",1,dirs[0]);
		TestCase.assertEquals("optimal dir",0,dirs[1]);
		ray.setStart(new Float[]{0.9f,0.9f});
		ray.setDir(new Float[]{0.1f,0.099f});
		math.normalize(ray.getDir());
		lattice.getOptDirPerm(ray,dirs);
		TestCase.assertEquals("optimal dir",0,dirs[0]);
		TestCase.assertEquals("optimal dir",1,dirs[1]);
		ray.setStart(new Float[]{0.9f,0.9f});
		ray.setDir(new Float[]{0.1f,0.101f});
		math.normalize(ray.getDir());
		lattice.getOptDirPerm(ray,dirs);
		TestCase.assertEquals("optimal dir",1,dirs[0]);
		TestCase.assertEquals("optimal dir",0,dirs[1]);
		ray.setStart(new Float[]{0.9f,0.9f});
		ray.setDir(new Float[]{-0.1f,-0.099f});
		math.normalize(ray.getDir());
		lattice.getOptDirPerm(ray,dirs);
		TestCase.assertEquals("optimal dir",0,dirs[0]);
		TestCase.assertEquals("optimal dir",1,dirs[1]);
		ray.setStart(new Float[]{0.9f,0.9f});
		ray.setDir(new Float[]{-0.1f,-0.101f});
		math.normalize(ray.getDir());
		lattice.getOptDirPerm(ray,dirs);
		TestCase.assertEquals("optimal dir",1,dirs[0]);
		TestCase.assertEquals("optimal dir",0,dirs[1]);
		ray.setStart(new Float[]{0.9f,0.6f});
		ray.setDir(new Float[]{0.1f,-0.099f});
		math.normalize(ray.getDir());
		lattice.getOptDirPerm(ray,dirs);
		TestCase.assertEquals("optimal dir",0,dirs[0]);
		TestCase.assertEquals("optimal dir",1,dirs[1]);
		ray.setStart(new Float[]{0.9f,0.6f});
		ray.setDir(new Float[]{0.1f,-0.101f});
		math.normalize(ray.getDir());
		lattice.getOptDirPerm(ray,dirs);
		TestCase.assertEquals("optimal dir",1,dirs[0]);
		TestCase.assertEquals("optimal dir",0,dirs[1]);
		ray.setStart(new Float[]{0.6f,0.9f});
		ray.setDir(new Float[]{-0.1f,0.099f});
		math.normalize(ray.getDir());
		lattice.getOptDirPerm(ray,dirs);
		TestCase.assertEquals("optimal dir",0,dirs[0]);
		TestCase.assertEquals("optimal dir",1,dirs[1]);
		ray.setStart(new Float[]{0.6f,0.9f});
		ray.setDir(new Float[]{-0.1f,0.101f});
		math.normalize(ray.getDir());
		lattice.getOptDirPerm(ray,dirs);
		TestCase.assertEquals("optimal dir",1,dirs[0]);
		TestCase.assertEquals("optimal dir",0,dirs[1]);
	}
	private <T extends Number>void assertEquals(String msg,T val,T res,T diff,VecMath<T> m) {
		if(m.cmp(m.abs(m.sub(val,res)),diff)>0) {
			System.out.println("Test failed");
			System.out.println(msg);
			System.out.println("Expected: "+val);
			System.out.println("Got: "+res);
			System.out.println("Difference: "+m.sub(val,res));
			System.out.println("Max Difference: "+diff);
			Assert.assertEquals(msg,true,false);
			System.exit(0);
		}
	}
}
